class LgException(Exception):
    def __init__(self, message: str) -> None:
        super().__init__(message)


class LgParameterListLimitException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"ERROR - {message}")


class LgInconsistencyException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"LG INCONSISTENCY - {message}")


class LgErrorException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"LG ERROR - {message}")


class LgNotProcessException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"LG NOT PROCESS - {message}")


class LgTaskExecutionException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"LG TASK EXECUTION ERROR - {message}")


class LgTaskCancelledException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"LG TASK CANCELLED - {message}")


class LgTaskNotRespondingException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"LG TASK NOT RESPONDING - {message}")


class LgTaskCompletedWithInconsistenciesException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"LG TASK COMPLETED WITH INCONSISTENCIES - {message}")


class LgTaskNotCompletedYetException(LgException):
    def __init__(self, message: str) -> None:
        super().__init__(f"LG TASK NOT COMPLETED YET - {message}")



from typing import Literal

from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import (
    LgApiReturn,
    LgApiPaginationReturn,
    LgApiExecReturn
)
from lg_payroll_api.helpers.base_client import (
    BaseLgServiceClient,
    LgAuthentication
)
from lg_payroll_api.utils.enums import (
    get_enum_value,
    EnumTipoStatus,
    EnumTipoPonto,
    EnumGrauDeInstrucao,
    EnumMesesExperiencia,
    EnumModeloPosicao
)
from datetime import date


class LgApiPositionClient(BaseLgServiceClient):
    """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

    Default class to connect with the role endpoints, service "v1/ServicoDePosicao"
    """

    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v1/ServicoDePosicao")
    
    @staticmethod
    def __bool_to_int(value: bool) -> int:
        if isinstance(value, bool):
            value = int(value)

        return value

    def consult_list(
        self,
        company_code: int,
        records_limit: int = 50,
        permission_to_recorded_employee: bool = None,
        only_actives: bool = None,
        consider_freezed: bool = None,
        only_officials: bool = None,
        not_temporaries: bool = None,
        search_term: str = None
    ) -> LgApiReturn:
        """Consult list of positions.

        Args:
            company_code (int, mandatory): The company code
            records_limit (int, optional): Quantity of records to return in requisition, 50 by default
            permission_to_recorded_employee (bool, optional): Flag to return only positions with permition
            to recorded employees
            only_actives (bool, optional): Flag to return only active positions
            consider_freezed (bool, optional): Flag to consider freezed positions
            only_officials (bool, optional): Flag to return only official positions
            not_temporaries (bool, optional): Flag to not return temporaries positions
            search_term (str, optional): Search term
        """
        permission_to_recorded_employee = self.__bool_to_int(permission_to_recorded_employee)
        only_actives = self.__bool_to_int(only_actives)
        consider_freezed = self.__bool_to_int(consider_freezed)
        only_officials = self.__bool_to_int(only_officials)
        not_temporaries = self.__bool_to_int(not_temporaries)

        params = {
            "QuantidadeDeItensRetornados": records_limit,
            "ComPermissaoParaCadastrarColaborador": permission_to_recorded_employee,
            "SomenteAtivos": only_actives,
            "ConsiderarPosicoesCongeladas": consider_freezed,
            "ConsiderarPosicoesApenasDoTipoOficial": only_officials,
            "ConsiderarPosicoesDiferentesDoTipoTemporaria": not_temporaries,
            "TermoDeBusca": search_term,
            "Empresa": {"Codigo": company_code}
        }

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarLista,
                    body=params,
                    parse_body_on_request=False
                )
            )
        )

    def consult_list_by_demand(
        self,
        company_code: int,
        search_term: str = None,
        permission_to_recorded_employee: bool = None,
        only_actives: bool = None,
        consider_freezed: bool = None,
        only_officials: bool = None,
        not_temporaries: bool = None,
    ) -> LgApiPaginationReturn:
        """Consult list of positions on demand.

        Args:
            company_code (int, mandatory): The company code
            permission_to_recorded_employee (bool, optional): Flag to return only positions with permition
            to recorded employees
            only_actives (bool, optional): Flag to return only active positions
            consider_freezed (bool, optional): Flag to consider freezed positions
            only_officials (bool, optional): Flag to return only official positions
            not_temporaries (bool, optional): Flag to not return temporaries positions
            search_term (str, optional): Search term
        """
        permission_to_recorded_employee = self.__bool_to_int(permission_to_recorded_employee)
        only_actives = self.__bool_to_int(only_actives)
        consider_freezed = self.__bool_to_int(consider_freezed)
        only_officials = self.__bool_to_int(only_officials)
        not_temporaries = self.__bool_to_int(not_temporaries)

        params = {
            "SomenteComPermissaoParaCadastrarColaborador": permission_to_recorded_employee,
            "SomenteAtivos": only_actives,
            "ConsiderarPosicoesCongeladas": consider_freezed,
            "ConsiderarPosicoesApenasDoTipoOficial": only_officials,
            "ConsiderarPosicoesDiferentesDoTipoTemporaria": not_temporaries,
            "TermoDaBusca": search_term,
            "Empresa": {"Codigo": company_code}
        }

        return LgApiPaginationReturn(
            auth=self.lg_client,
            wsdl_service=self.wsdl_client,
            service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
            body=params,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
                    body=params,
                    parse_body_on_request=False
                )
            )
        )
    
    def consult(self, code: int) -> LgApiReturn:
        """Retrieve a position by code.

        Args:
            code (int, mandatory): LG position code
        """
        params = {"Codigo": code}
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.Consultar,
                    body=params,
                    parse_body_on_request=False
                )
            )
        )
    
    def save(
        self,
        description: str,
        status: EnumTipoStatus,
        start_date: date = None,
        type: int = None,
        reason_code: int = None,
        code: int = None,
        end_date: date = None,
        jobs_quantity: int = None,
        enable_employee_enrollment: bool = None,
        is_key: bool = False,
        freeze: bool = False,
        freeze_end_date: date = None,
        company_code: int = None,
        office_code: int = None,
        work_local_code: str = None,
        tax_jurisdiction_code: int = None,
        cost_center_code: str = None,
        professional_nature_code: int = None,
        responsible_authority_code: int = None,
        permanent_role_code: int = None,
        commissioned_role_code: int = None,
        union_code: int = None,
        proffessional_liberal_union_code: int = None,
        organizational_unit_code: int = None,
        unique_combination: bool = False,
        timekeeping_required: bool = False,
        timekeeping_type: EnumTipoPonto = None,
        work_shift_code: str = None,
        function_code: int = None,
        commissioned_function_code: int = None,
        areas_of_expertise_list: list[int] = None,
        hierarchical_level_code: int = None,
        short_description: str = None,
        long_description: str = None,
        education_level: EnumGrauDeInstrucao = None,
        education_notes: str = None,
        experience: EnumMesesExperiencia = None,
        experience_notes: str = None,
        competences_group_list: list[tuple[int, int]] = None,
        competences_profile_list: list[tuple[int, int, int]] = None,
        replicate_positions: bool = None,
        replication_context: EnumModeloPosicao = None,
        original_position_transfer_employees: bool = None,
        original_position_code: int = None,
        original_position_status: EnumTipoStatus = None,
        original_position_description: str = None,
    ) -> LgApiReturn:
        """Save position on LG system.

        Args:
            code (int, mandatory): LG position code 
            description (str, mandatory): Description of the position, like a name
            status (EnumTipoStatus, mandatory): Status of position, if it is active, inactive and others
            start_date (date, mandatory): Start date of position. Can be used to set another record on
                position changes history
            type (int, mandatory): Type of position code
            reason_code (int, mandatory): Reason of position code
            end_date (date, optional): End date of the position
            jobs_quantity (int, optional): Quantity of jobs available in that position
            enable_employee_enrollment (bool, optional): Flag to enable or disable employee enrollment at
                that position
            is_key (bool, optional): Flag if this is a key position
            freeze (bool, optional): Flag to freeze or defreeze position
            freeze_end_date (date, optional): End date of freezing
            company_code (int, mandatory): Company code of the position
            office_code (int, optional): Office code of the position
            work_local_code (str, optional): Work local code
            tax_jurisdiction_code (int, optional): The tax jurisdiction code
            cost_center_code (str, optional): Cost center code
            professional_nature_code (int, optional): Professional nature code
            responsible_authority_code (int, optional): Responsible Authority code
            permanent_role_code (int, optional): Position role code
            commissioned_role_code (int, optional): Position commissioned role code
            union_code (int, optional): Union code
            proffessional_liberal_union_code (int, optional): Professional unit code
            organizational_unit_code (int, optional): Organizational unit code
            unique_combination (bool, optional): Flag if this position have unique combination. This will prevent
                that another positions to have the same composition of this one
            timekeeping_required (bool, optional): Flag if timekeeping is required or not to this position
            timekeeping_type (EnumTipoPonto, optional): Type of timekeeping
            work_shift_code (str, optional): Work shift code
            function_code (int, optional): Function code
            commissioned_function_code (int, optional): Commissioned function code
            areas_of_expertise_list (list[int], optional): List of areas of expertise codes
            hierarchical_level_code (int, optional): Hierarchical level code
            short_description (str, optional): Short description of this position
            long_description (str, optional): Long description of this position
            education_level (EnumGrauDeInstrucao, optional): Education level of this position
            education_notes (str, optional): Education notes of this position
            experience (EnumMesesExperiencia, optional): Experience time of this position
            experience_notes (str, optional): Experience notes of this position
            competences_group_list (list[tuple[int, int]], optional): Competences group of this position. To
                define it, you need to pass a list of tuples containing `[(competence_group_code, percent)]`
            competences_profile_list (list[tuple[int, int, int]], optional): List of competences profiles. To
                define it, you need to pass a list of tuples containing `[(weight, competence_code, competence_group_code)]`
            replicate_positions (bool, optional): Flag to create replicated position
            replication_context (EnumModeloPosicao, optional): Identify if the replica is temporary or dupplicated
            original_position_transfer_employees (bool, optional): Indicates if the employees in original position
                have to be transfered to replica position
            original_position_code (int, optional): Code of original position
            original_position_status (EnumTipoStatus, optional): Status of original position
            original_position_description (str, optional): Description of original position
        """
        date_frmt = "%Y-%m-%d"
        if isinstance(start_date, date):
            start_date = start_date.strftime(date_frmt)

        if isinstance(end_date, date):
            end_date = end_date.strftime(date_frmt)

        if isinstance(freeze_end_date, date):
            freeze_end_date = freeze_end_date.strftime(date_frmt)

        enable_employee_enrollment = self.__bool_to_int(enable_employee_enrollment)
        is_key = self.__bool_to_int(is_key)
        freeze = self.__bool_to_int(freeze)
        unique_combination = self.__bool_to_int(unique_combination)
        timekeeping_required = self.__bool_to_int(timekeeping_required)
        original_position_transfer_employees = self.__bool_to_int(original_position_transfer_employees)

        status = get_enum_value(status)
        timekeeping_type = get_enum_value(timekeeping_type)
        education_level = get_enum_value(education_level)
        experience = get_enum_value(experience)
        replication_context = get_enum_value(replication_context)
        original_position_status = get_enum_value(original_position_status)

        params = {
            "DataInicial": start_date,
            "DataFinal": end_date,
            "TipoDaPosicao": {"Codigo": type},
            "MotivoDaPosicao": {"Codigo": reason_code},
            "ComposicaoDaPosicao": {
                "CodigoEmpresa": company_code,
                "CodigoEstabelecimento": office_code,
                "CodigoLocalDeTrabalho": work_local_code,
                "CodigoLotacaoTributaria": tax_jurisdiction_code,
                "CodigoCentroDeCusto": cost_center_code,
                "CodigoNaturezaProfissional": professional_nature_code,
                "CodigoOrgaoResponsavel": responsible_authority_code,
                "CodigoCargoEfetivo": permanent_role_code,
                "CodigoCargoComissionado": commissioned_role_code,
                "CodigoSindicato": union_code,
                "CodigoSindicatoProfissionalLiberal": proffessional_liberal_union_code,
                "CodigoUnidadeOrganizacional": organizational_unit_code,
                "CombinacaoUnica": unique_combination,
                "MarcaPonto": timekeeping_required,
                "TipoDePonto": timekeeping_type,
                "CodigoEscala": work_shift_code,
                "CodigoFuncao": function_code,
                "CodigoFuncaoComissionada": commissioned_function_code,
            },
            "PermiteCadastrarColaborador": enable_employee_enrollment,
            "EhChave": is_key,
            "CongelarPosicao": freeze,
            "DataFinalCongelamentoDaPosicao": freeze_end_date,
            "ListaDeAreaDeAtuacao": {
                "AreaAtuacaoPosicao": (
                    [{"Codigo": code} for code in areas_of_expertise_list]
                        if areas_of_expertise_list else None
                )
            },
            "NivelHierarquico": {"Codigo": hierarchical_level_code},
            "QuantidadeVagas": jobs_quantity,
            "Perfil": {
                "DadosGerais": {
                    "DescricaoResumida": short_description,
                    "DescricaoDetalhada": long_description,
                    "GrauDeInstrucao": education_level,
                    "ObservacaoEscolaridade": education_notes,
                    "Experiencia": experience,
                    "ObservacaoExperiencia": experience_notes
                },
                "ListaPerfilGrupoDeCompetencias": {
                    "PerfilGrupoDeCompetenciasPosicao": (
                        [
                            {
                                "CodigoGrupoDeCompetencias": competence_group_code,
                                "Percentual": percent
                            } for competence_group_code, percent in competences_group_list
                        ] if competences_group_list else None
                    )
                },
                "ListaPerfilCompetencia": {
                    "PerfilCompetenciaPosicao": (
                        [
                            {
                                "Peso": weight,
                                "CodigoDaCompetencia": competence_code,
                                "CodigoDoGrupoDaCompetencia": competence_group_code
                            } for weight, competence_code, competence_group_code in competences_profile_list
                        ] if competences_profile_list else None
                    )
                }
            },
            "ReplicarPosicoes": {
                "PosicaoOriginal": {
                    "Descricao": original_position_description,
                    "Status": original_position_status,
                    "Codigo": original_position_code
                },
                "TransferirColaboradores": original_position_transfer_employees,
                "Contexto": replication_context
            } if replicate_positions else None,
            "Descricao": description,
            "Status": status,
            "Codigo": code
        }

        return LgApiExecReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.Salvar,
                    body=params,
                    parse_body_on_request=False
                )
            )
        )
    
    def delete(
        self,
        code: int,
        historical_start_date: date = None,
    ) -> LgApiExecReturn:
        """Delete a position

        Args:
            code (int, mandatory): Code of LG position
            historical_start_date (date, optional): The start date of historic record
        """
        params = {
            "DataInicial": historical_start_date.strftime("%Y-%m-%d"),
            "Codigo": code
        }
        return LgApiExecReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.Excluir,
                    body=params,
                    parse_body_on_request=False
                )
            )
        )



from datetime import date
from typing import Literal

from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import (
    LgApiExecutionReturn,
    LgApiPaginationReturn,
    LgApiReturn,
)
from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication
from lg_payroll_api.utils.enums import (
    SITUATIONS,
    EnumCampoDeBuscaDoContratoDeTrabalho,
    EnumTipoDeDadosModificados,
    EnumTipoDeOperacaoContratoLog,
)
from lg_payroll_api.utils.lg_exceptions import LgParameterListLimitException


class LgApiEmploymentContract(BaseLgServiceClient):
    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v2/ServicoDeContratoDeTrabalho")

    def consult(self, contract_code: str, company_code: int) -> LgApiReturn:
        body = {
            "Colaborador": {
                "Matricula": contract_code,
                "Empresa": {"Codigo": company_code},
            }
        }

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.Consultar,
                    body=body,
                )
            )
        )

    def consult_list(
        self,
        company_code: int,
        search_field: EnumCampoDeBuscaDoContratoDeTrabalho = None,
        search_value: str = None,
        employee_type: Literal["Funcionário", "Autônomo"] = None,
    ) -> LgApiReturn:
        if (
            search_field == None
            and not search_value == None
            or search_value == None
            and not search_field == None
        ):
            raise ValueError(
                "If search field is defined, you need to define a search value or vice versa."
            )

        body = {
            "Empresa": {"FiltroComCodigoNumerico": {"Codigo": company_code}},
            "CampoDeBusca": search_field,
            "TipoDoColaborador": employee_type,
            "TermoDeBusca": search_value,
        }
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarLista,
                    body=body,
                )
            )
        )

    def consult_list_by_company(
        self,
        company_code: int,
        contracts_codes: list[str],
    ) -> LgApiReturn:

        if len(contracts_codes) > 50:
            raise LgParameterListLimitException(
                "Person ids list has exceeded the limit of 50 items."
            )

        body = {
            "CodigoDaEmpresa": company_code,
            "ListaDeMatriculas": {"string": contracts_codes},
        }
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaPorEmpresa,
                    body=body,
                )
            )
        )

    def list_on_demand(
        self,
        companies: list[int] = None,
        offices: list[tuple[int, int]] = None,
        employee_status: list[SITUATIONS] = None,
        current_page: int = None,
    ) -> LgApiPaginationReturn:
        body = {
            "Empresas": [
                {"FiltroComCodigoNumerico": {"Codigo": company}}
                for company in companies
            ]
            if companies
            else None,
            "Estabelecimentos": [
                {
                    "FiltroComCodigoNumericoEEmpresa": {
                        "Codigo": office[0],
                        "Empresa": {"Codigo": office[1]},
                    }
                }
                for office in offices
            ]
            if offices
            else None,
            "TiposDeSituacoes": [{"int": situation} for situation in employee_status]
            if employee_status
            else None,
            "PaginaAtual": current_page,
        }
        return LgApiPaginationReturn(
            auth=self.lg_client,
            wsdl_service=self.wsdl_client,
            service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
            body=body,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
                    body=body,
                )
            )
        )

    def consult_manager_list(
        self,
        employee_code: str,
        employee_company_id: int,
        situations_types: list[SITUATIONS] = None,
    ) -> LgApiReturn:
        body = {
            "TiposDeSituacoes": situations_types,
            "Colaborador": {
                "Matricula": employee_code,
                "Empresa": {"Codigo": employee_company_id},
            },
        }
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaDeGestorImediato,
                    body=body,
                )
            )
        )

    def list_changed_contracts(
        self,
        company_code: int,
        start_date: date,
        end_date: date,
        operations_types: list[EnumTipoDeOperacaoContratoLog] = [
            EnumTipoDeOperacaoContratoLog.INCLUSAO.value,
            EnumTipoDeOperacaoContratoLog.ALTERACAO.value,
            EnumTipoDeOperacaoContratoLog.EXCLUSAO.value,
        ],
        situation_type: list[SITUATIONS] = None,
        modified_data_type: EnumTipoDeDadosModificados = EnumTipoDeDadosModificados.CONTRATUAIS_E_PESSOAIS.value,
        enrollments: list[str] = None,
    ) -> LgApiReturn:
        body = {
            "filtro": {
                "TiposDeSituacao": situation_type,
                "TipoDeDadosModificados": modified_data_type,
                "TiposDeOperacoes": [
                    {"TipoDeOperacao": {"Valor": operation}}
                    for operation in operations_types
                ],
                "CodigoDaEmpresa": company_code,
                "ListaDeMatriculas": enrollments,
                "PeriodoDeBusca": {
                    "DataInicio": start_date.strftime("%Y-%m-%d"),
                    "DataFim": end_date.strftime("%Y-%m-%d"),
                },
            }
        }
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaDeModificados,
                    body=body,
                    parse_body_on_request=True,
                )
            )
        )

    def insert_manager(
        self,
        employee_code: str,
        employee_company_id: int,
        manager_code: str,
        manager_company_id: int,
        start_date: date = None,
        end_date: date = None,
    ) -> LgApiExecutionReturn:
        body = {
            "ListaDeAssociacaoContratoGestor": {
                "FiltroDeAssociacaoContratoGestor": {
                    "Contrato": {
                        "Matricula": employee_code,
                        "Empresa": {
                            "Codigo": employee_company_id,
                        },
                    },
                    "Gestores": {
                        "FiltroComIdentificacaoDeContratoV2": {
                            "Matricula": manager_code,
                            "Empresa": {"Codigo": manager_company_id},
                        }
                    },
                }
            },
            "Periodo": {
                "DataInicio": start_date.strftime("%Y-%m-%d") if start_date else None,
                "DataFim": end_date.strftime("%Y-%m-%d") if end_date else None,
            },
        }
        return LgApiExecutionReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.InserirGestoresNaFicha,
                    body=body,
                )
            )
        )

    def delete_manager(
        self,
        employee_code,
        employee_company_id,
        manager_code,
        manager_company_id,
        start_date=None,
        end_date=None,
    ) -> LgApiExecutionReturn:
        body = {
            "ListaDeAssociacaoContratoGestor": {
                "FiltroDeAssociacaoContratoGestor": {
                    "Contrato": {
                        "Matricula": employee_code,
                        "Empresa": {"Codigo": employee_company_id},
                    },
                    "Gestores": {
                        "FiltroComIdentificacaoDeContratoV2": {
                            "Matricula": manager_code,
                            "Empresa": {"Codigo": manager_company_id},
                        }
                    },
                }
            },
            "Periodo": {
                "DataInicio": start_date if start_date else None,
                "DataFim": end_date if end_date else None,
            },
        }
        return LgApiExecutionReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ExcluirGestoresNaFicha,
                    body=body,
                )
            )
        )



from lg_payroll_api.scripts.company import LgApiCompanyClient
from lg_payroll_api.scripts.cost_center import LgApiCostCenterClient
from lg_payroll_api.scripts.employee import LgApiEmployee
from lg_payroll_api.scripts.employment_contract import LgApiEmploymentContract
from lg_payroll_api.scripts.organizational_unit import LgApiOrganizationalUnitClient
from lg_payroll_api.scripts.reports import LgReportServiceClient, ReportParameters
from lg_payroll_api.scripts.role import LgApiRoleClient
from lg_payroll_api.scripts.work_scale import LgApiWorkScaleClient
from lg_payroll_api.scripts.additional_information_value import LgApiAdditionalInformationValueClient
from lg_payroll_api.scripts.additional_information import LgApiAdditionalInformationClient
from lg_payroll_api.scripts.position import LgApiPositionClient
from lg_payroll_api.scripts.movement import LgApiMovementClient



from datetime import date
from typing import Literal

from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import LgApiPaginationReturn, LgApiReturn
from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication
from lg_payroll_api.utils.enums import (
    EnumTipoDeDadosModificadosDaUnidadeOrganizacional,
    EnumTipoDeOperacao,
)


class LgApiOrganizationalUnitClient(BaseLgServiceClient):
    """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

    Default class to connect with the organizational unit endpoints
    """

    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(
            lg_auth=lg_auth, wsdl_service="v1/ServicoDeUnidadeOrganizacional"
        )

    def consult_list(
        self,
        company_code: int,
        level: int = None,
        only_normal: Literal[0, 1] = None,
        only_actives: Literal[0, 1] = None,
        only_with_employees_registration_available: Literal[0, 1] = None,
        search_term: str = None,
    ) -> LgApiReturn:
        """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

        Endpoint to get list of organizational units in LG System

        Returns:
            LgApiReturn: A List of OrderedDict that represents an Object(RetornoDeConsultaLista<UnidadeOrganizacionalParcial>) API response
                [
                    Tipo : int
                    Mensagens : [string]
                    CodigoDoErro : string
                    Retorno : list[Object(UnidadeOrganizacionalParcial)]
                ]
        """

        params = {
            "Nivel": level,
            "SomenteNormais": only_normal,
            "SomenteAtivos": only_actives,
            "SomenteComPermissaoParaCadColaborador": only_with_employees_registration_available,
            "TermoDeBusca": search_term,
            "Empresa": {
                "FiltroComCodigoNumerico": {
                    "Codigo": company_code,
                }
            },
        }

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarLista,
                    body=params,
                    parse_body_on_request=False,
                )
            )
        )

    def list_on_demand(
        self,
        company_code: int = None,
        level: int = None,
        only_actives: Literal[0, 1] = None,
        only_with_employees_registration_available: Literal[0, 1] = None,
        search_term: str = None,
        page: int = None,
    ) -> LgApiPaginationReturn:
        """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

        Endpoint to get list on demand of organizational units in LG System

        Returns:

        A List of OrderedDict that represents an Object(RetornoDeConsultaListaPorDemanda<UnidadeOrganizacional>) API response
            [
                Tipo : int
                Mensagens : [string]
                CodigoDoErro : string
                UnidadeOrganizacional : list[Object(UnidadeOrganizacional)]
            ]
        """

        params = {
            "FiltroDeUnidadeOrganizacionalPorDemanda": {
                "Nivel": level,
                "SomenteAtivos": only_actives,
                "SomenteComPermissaoParaCadColaborador": only_with_employees_registration_available,
                "TermoDeBusca": search_term,
                "Empresa": {
                    "FiltroComCodigoNumerico": {
                        "Codigo": company_code,
                    }
                }
                if company_code
                else None,
                "PaginaAtual": page,
            }
        }

        return LgApiPaginationReturn(
            auth=self.lg_client,
            wsdl_service=self.wsdl_client,
            service_client=self.wsdl_client.service.ConsulteListaPorDemanda,
            body=params,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsulteListaPorDemanda,
                    body=params,
                )
            )
        )

    def consult_changed_list(
        self,
        start_date: date,
        end_date: date,
        operation_types: list[EnumTipoDeOperacao] = [
            EnumTipoDeOperacao.ALTERACAO.value,
            EnumTipoDeOperacao.INCLUSAO.value,
            EnumTipoDeOperacao.EXCLUSAO.value,
        ],
        organizational_units_codes: list[int] = None,
        modified_data_type: EnumTipoDeDadosModificadosDaUnidadeOrganizacional = None,
        consult_inferior_organizational_units: Literal[0, 1] = 0,
    ) -> LgApiReturn:
        """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

        Endpoint to get list of organizational units changed in LG System

        Returns:

        A List of OrderedDict that represents an Object(RetornoDeConsultaListaPorDemanda<UnidadeOrganizacional>) API response
            [
                Tipo : int
                Mensagens : [string]
                CodigoDoErro : string
                UnidadeOrganizacional : list[Object(UnidadeOrganizacional)]
            ]
        """

        params = {
            "filtro": {
                "ListaDeCodigos": organizational_units_codes,
                "TipoDeDadosModificados": modified_data_type,
                "ConsultarUnidadesOrganizacionaisInferiores": consult_inferior_organizational_units,
                "TiposDeOperacoes": [
                    {"Operacao": {"Valor": operation}} for operation in operation_types
                ],
                "PeriodoDeBusca": {
                    "DataInicio": start_date.strftime("%Y-%m-%d"),
                    "DataFim": end_date.strftime("%Y-%m-%d"),
                },
            }
        }

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaDeUnidadesOrganizacionaisModificadas,
                    body=params,
                    parse_body_on_request=True,
                )
            )
        )



from typing import Union
from requests import Session
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
from zeep import Client, Transport
from zeep.plugins import HistoryPlugin
from lg_payroll_api.utils.settings import LG_API_DTO
from lg_payroll_api.helpers.authentication import LgAuthentication
from lg_payroll_api.utils.aux_functions import clean_none_values_dict


class BaseLgServiceClient:
    """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

    Default class to send requests to LG Soap API services
    """

    def __init__(
        self,
        lg_auth: LgAuthentication,
        wsdl_service: Union[str, Client],
        requests_history: bool = False,
    ):
        super().__init__()
        self.requests_history = HistoryPlugin() if requests_history else None
        self.lg_client = lg_auth
        self.lg_dto = LG_API_DTO
        if isinstance(wsdl_service, Client):
            self.wsdl_client: Client = wsdl_service

        elif isinstance(wsdl_service, str):
            session = Session()
            adapter = HTTPAdapter(
                pool_connections=20,
                pool_maxsize=20,   
                max_retries=Retry(
                    total=5,           
                    backoff_factor=0.5,
                    status_forcelist=[502, 503, 504],
                )
            )
            session.mount("https://", adapter)
            self.wsdl_client: Client = Client(
                wsdl=f"{self.lg_client.base_url}/{wsdl_service}?wsdl",
                plugins=[self.requests_history] if self.requests_history else None,
                transport=Transport(session=session),
            )

        else:
            raise ValueError("Wsdl must be zeep Client or String.")

    def send_request(
        self,
        service_client: Client,
        body: dict,
        parse_body_on_request: bool = False,
        send_none_values: bool = False,
    ):
        if not send_none_values:
            body = clean_none_values_dict(body)

        if parse_body_on_request:
            response = service_client(**body, _soapheaders=self.lg_client.auth_header)

        else:
            response = service_client(body, _soapheaders=self.lg_client.auth_header)

        return response



from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import LgApiPaginationReturn, LgApiReturn, LgApiExecutionReturn
from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication
from lg_payroll_api.utils.enums import (
    EnumTipoEntidadeInformacaoAdicional,
    EnumTipoDeInformacaoAdicional
)


class LgApiAdditionalInformationClient(BaseLgServiceClient):
    """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

    Default class to connect with the additional information endpoints
    """
    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(
            lg_auth=lg_auth, wsdl_service="v1/ServicoDeInformacaoAdicional"
        )

    def consult_list_on_demand(
        self,
        concept_code: EnumTipoEntidadeInformacaoAdicional,
        group_code: int = None,
        type: EnumTipoDeInformacaoAdicional = None,
    ) -> LgApiPaginationReturn:
        """Consult list of additional information fields on demand
        """
        if isinstance(concept_code, EnumTipoEntidadeInformacaoAdicional):
            concept_code = concept_code.value

        if isinstance(type, EnumTipoDeInformacaoAdicional):
            type = type.value

        params = {
            
            "ConceitoDaInformacaoAdicional": concept_code,
            "CodigoDoGrupo": group_code,
            "Tipo": type,
        }

        return LgApiPaginationReturn(
            auth=self.lg_client,
            wsdl_service=self.wsdl_client,
            service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
            body=params,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
                    body=params,
                )
            )
        )



# This module contais a authentication class


class LgAuthentication:
    base_url: str = None
    retry_time_request: int = None

    def __init__(
        self,
        base_url: str,
        user: str,
        password: str,
        guild_tenant: str,
        environment_context: int,
    ) -> None:
        """Authentication class used to generate a auth header dictionary

        Args:
            base_url (str, mandatory): Your LG Environment Url, for example "https://your-domain-api.lgcloud.com.br"
            user (str, mandatory): An user able to use LG API, for example "your_user@domain.com"
            password (str, mandatory): Password of user
            guid_tenant (str, mandatory): This is a client identification guid. You can get this code by accessing the portal Gent.te. See more about this in https://portalgentedesucesso.lg.com.br/documentacao.api.suitegente/Autenticacao
            environment_context: (int, mandatory): This is a client identification environment code. You can get this code in the same way of guid_tenant
        """
        self.base_url = base_url
        self.user = user
        self.password = password
        self.guild_tenant = guild_tenant
        self.environment_context = environment_context

    @property
    def auth_header(self) -> dict:
        """Return a auth header to xml body in json format."""

        environment: dict = {"Ambiente": self.environment_context}

        header_authentication = {
            "TokenUsuario": {
                "Senha": self.password,
                "Usuario": self.user,
                "GuidTenant": self.guild_tenant,
            }
        }

        return {
            "LGContextoAmbiente": environment,
            "LGAutenticacao": header_authentication,
        }



# This module contains auxiliary functions to transform data
import json
import re
from os import path
from typing import Union


def clean_none_values_dict(data: dict) -> dict:
    """Remove none fields from dictionary."""

    result: dict = {}
    for key, value in data.items():
        if value != None:
            if isinstance(value, dict):
                result[key] = clean_none_values_dict(value)
                if result[key] == {}:
                    result[key] = None

            elif isinstance(value, list):
                if len(value) > 0:
                    if isinstance(value[0], dict):
                        result[key] = [clean_none_values_dict(item) for item in value]
                        if result[key] == []:
                            result[key] = None

                    else:
                        result[key] = value

                else:
                    result[key] = value

            else:
                result[key] = value

    return result


def extract_file_extension(file_name: str) -> str:
    pattern = r"\.([a-zA-Z0-9]+)$"
    match = re.search(pattern, file_name)

    if match:
        return match.group(1)

    else:
        return None


def read_json_file(file_path: str) -> Union[dict, list[dict]]:
    if not path.exists(file_path):
        raise FileNotFoundError(f"File not found. {file_path}")

    with open(file_path, "r") as f:
        data = json.loads(f.read())

    return data


def bool_to_int(value: bool) -> int:
    if isinstance(value, bool):
        value = int(value)

    return value



from lg_payroll_api.helpers import LgAuthentication
from lg_payroll_api.scripts import (
    LgApiCompanyClient,
    LgApiCostCenterClient,
    LgApiEmployee,
    LgApiEmploymentContract,
    LgApiOrganizationalUnitClient,
    LgReportServiceClient,
    ReportParameters,
    LgApiRoleClient,
    LgApiWorkScaleClient,
    LgApiAdditionalInformationValueClient,
    LgApiAdditionalInformationClient,
    LgApiPositionClient,
    LgApiMovementClient
)


class LgPayrollApi:
    """Interface to access endpoints."""

    def __init__(self, auth: LgAuthentication) -> None:
        self.__auth: LgAuthentication = auth

    @property
    def cost_center_service(self) -> LgApiCostCenterClient:
        """Access cost centers service methods."""
        return LgApiCostCenterClient(self.__auth)

    @property
    def employment_contract_service(self) -> LgApiEmploymentContract:
        """Access employment contracts service methods."""
        return LgApiEmploymentContract(self.__auth)

    @property
    def organizational_unit_service(self) -> LgApiOrganizationalUnitClient:
        """Access organizational units service methods."""
        return LgApiOrganizationalUnitClient(self.__auth)

    @property
    def company_service(self) -> LgApiCompanyClient:
        """Access companies service methods."""
        return LgApiCompanyClient(self.__auth)

    @property
    def employee_service(self) -> LgApiEmployee:
        """Access employee service methods."""
        return LgApiEmployee(self.__auth)

    @property
    def report_service(self) -> LgReportServiceClient:
        """Access report generator service methods."""
        return LgReportServiceClient(self.__auth)

    @property
    def role_service(self) -> LgApiRoleClient:
        """Access role service methods."""
        return LgApiRoleClient(self.__auth)

    @property
    def work_scale_service(self) -> LgApiWorkScaleClient:
        """Access work scale service methods."""
        return LgApiWorkScaleClient(self.__auth)
    
    @property
    def additional_information_value(self) -> LgApiAdditionalInformationValueClient:
        return LgApiAdditionalInformationValueClient(self.__auth)

    @property
    def additional_information(self) -> LgApiAdditionalInformationClient:
        return LgApiAdditionalInformationClient(self.__auth)
    
    @property
    def position_service(self) -> LgApiPositionClient:
        return LgApiPositionClient(self.__auth)

    @property
    def movement_service(self) -> LgApiMovementClient:
        """Access movement service methods."""
        return LgApiMovementClient(self.__auth)



import re
from dataclasses import InitVar, dataclass
from io import BytesIO, StringIO
from time import sleep, time
from typing import List, OrderedDict, Union

from requests import Response, get
from zeep import Client
from zeep.helpers import serialize_object

from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication
from lg_payroll_api.utils.enums import EnumOperacaoExecutada, EnumTipoDeRetorno
from lg_payroll_api.utils.lg_exceptions import (
    LgErrorException,
    LgInconsistencyException,
    LgNotProcessException,
    LgTaskCancelledException,
    LgTaskCompletedWithInconsistenciesException,
    LgTaskExecutionException,
    LgTaskNotCompletedYetException,
    LgTaskNotRespondingException,
)


@dataclass
class BaseLgApiReturn:
    """This dataclass represents a Lg Api Return object

    Attr:
        Tipo (EnumTipoDeRetorno): The returnal type code
        Mensagens (OrderedDict[str, List[str]]): Messages of requisition
        CodigoDoErro (str): Error code
    """

    Tipo: EnumTipoDeRetorno
    Mensagens: OrderedDict[str, List[str]]
    CodigoDoErro: str

    def __post_init__(self):
        self._raise_for_errors()

    @property
    def _unpacked_messages(self) -> str:
        return " && ".join([" || ".join(value) for value in self.Mensagens.values()])

    def _raise_for_errors(self) -> None:
        if self.Tipo == EnumTipoDeRetorno.ERRO:
            raise LgErrorException(self._unpacked_messages)

        elif self.Tipo == EnumTipoDeRetorno.INCONSISTENCIA:
            raise LgInconsistencyException(self._unpacked_messages)

        elif self.Tipo == EnumTipoDeRetorno.NAO_PROCESSADO:
            raise LgNotProcessException(self._unpacked_messages)


@dataclass
class LgApiReturn(BaseLgApiReturn):
    """This dataclass represents a Lg Api Return object

    Attr:
        Tipo (EnumTipoDeRetorno): The returnal type code
        Mensagens (OrderedDict[str, List[str]]): Messages of requisition
        CodigoDoErro (str): Error code
        Retorno (Union[dict, OrderedDict, List[dict], List[OrderedDict], None]): Requisition result value
    """

    Retorno: Union[
        dict,
        OrderedDict,
        List[dict],
        List[OrderedDict],
        None
    ]


@dataclass
class LgApiExecReturn(BaseLgApiReturn):
    OperacaoExecutada: int
    Codigo: Union[int, None] = None
    CodigoDeIntegracao: Union[str, None] = None


@dataclass
class LgApiSaveListReturn(BaseLgApiReturn):
    """This dataclass represents a Lg Api Save List Return object

    Attr:
        Tipo (EnumTipoDeRetorno): The returnal type code
        Mensagens (OrderedDict[str, List[str]]): Messages of requisition
        CodigoDoErro (str): Error code
        Retorno (Union[dict, OrderedDict, List[dict], List[OrderedDict], None]): Requisition result value
        OperacaoExecutada (EnumOperacaoExecutada): Code of execution type
    """

    ListaDeRetorno: Union[
        dict,
        OrderedDict,
        List[dict],
        List[OrderedDict],
        None
    ]
    IdentificadorDeOcorrencias: Union[
        dict,
        OrderedDict,
        List[dict],
        List[OrderedDict],
        None
    ] = None


@dataclass
class LgApiPaginationReturn(LgApiReturn):
    """This dataclass represents a Lg Api Pagination Return object

    Attr:
        Tipo (EnumTipoDeRetorno): The returnal type code
        Mensagens (OrderedDict[str, List[str]]): Messages of requisition
        CodigoDoErro (str): Error code
        Retorno (Union[dict, OrderedDict, List[dict], List[OrderedDict], None]): Requisition result value
        NumeroDaPagina (int): Number of page returned
        QuantidadePorPagina (int): Total number of pages
        TotalGeral (int): Total number of records
    """

    NumeroDaPagina: int
    QuantidadePorPagina: int
    TotalDePaginas: int = None
    TotalGeral: int = None
    auth: InitVar[LgAuthentication] = None
    wsdl_service: InitVar[Client] = None
    service_client: InitVar[Client] = None
    body: InitVar[dict] = None
    page_key: InitVar[str] = "PaginaAtual"

    def __post_init__(
        self,
        auth: LgAuthentication,
        wsdl_service: Client,
        service_client: Client,
        body: dict,
        page_key: str = "PaginaAtual",
    ):
        self.NumeroDaPagina += 1
        self.TotalDePaginas = (
            0 if not self.TotalDePaginas
            else self.TotalDePaginas
        )
        self.TotalDePaginas += 1
        self._base_lg_service = BaseLgServiceClient(
            lg_auth=auth, wsdl_service=wsdl_service
        )
        self._service_client: Client = service_client
        self._body = body
        self._page_key = page_key
        super().__post_init__()

    def __increment_result(self, result: OrderedDict):
        self.Tipo = result["Tipo"]
        self.Mensagens = result["Mensagens"]
        self._raise_for_errors()

        returnal = result["Retorno"]
        if isinstance(returnal, list):
            self.Retorno += returnal

        elif isinstance(returnal, dict) or isinstance(returnal, OrderedDict):
            for key, value in returnal.items():
                if isinstance(value, list):
                    self.Retorno[key] += value

                else:
                    raise ValueError(
                        """Is not possible to unpack "Retorno" to increment values."""
                    )

    def all(self) -> "LgApiPaginationReturn":
        while self.NumeroDaPagina <= (self.TotalDePaginas - 1):
            self.NumeroDaPagina += 1
            self._body[self._page_key] = self.NumeroDaPagina
            self.__increment_result(
                serialize_object(
                    self._base_lg_service.send_request(
                        service_client=self._service_client,
                        body=self._body,
                    )
                )
            )

        return self


@dataclass
class LgApiExecutionReturn(LgApiReturn):
    """This dataclass represents a Lg Api Executions Return object

    Attr:
        Tipo (EnumTipoDeRetorno): The returnal type code
        Mensagens (OrderedDict[str, List[str]]): Messages of requisition
        CodigoDoErro (str): Error code
        Retorno (Union[dict, OrderedDict, List[dict], List[OrderedDict], None]): Requisition result value
        OperacaoExecutada (EnumOperacaoExecutada): Code of execution type
        Codigo (str): Concept code
        CodigoDeIntegracao (str): Integration concept code
    """

    OperacaoExecutada: EnumOperacaoExecutada
    Codigo: str
    CodigoDeIntegracao: str


@dataclass
class LgApiAsyncConsultReturn(LgApiReturn):
    """This dataclass represents a Lg Api Async Consult Return Return object

    Attr:
        Tipo (EnumTipoDeRetorno): The returnal type code
        Mensagens (OrderedDict[str, List[str]]): Messages of requisition
        CodigoDoErro (str): Error code
        Retorno (Union[dict, OrderedDict, List[dict], List[OrderedDict], None]): Requisition result value
        StatusProcessamento(int): Processing status code.
    """

    StatusProcessamento: int

    def _raise_for_errors(self) -> None:
        super()._raise_for_errors()

        if self.StatusProcessamento == 3550:  # Execution error
            raise LgTaskExecutionException(self._unpacked_messages)

        elif self.StatusProcessamento == 3465:  # Task cancelled
            raise LgTaskCancelledException(self._unpacked_messages)

        elif self.StatusProcessamento == 14296:  # Task not responding
            raise LgTaskNotRespondingException(self._unpacked_messages)

        elif self.StatusProcessamento == 39470:  # Task completed with errors
            raise LgTaskCompletedWithInconsistenciesException(self._unpacked_messages)

    def check_processing_completed(self) -> bool:
        """Check if task process if completed."""
        return self.StatusProcessamento == 3551

    def request_file(self, encoding: str = "ISO-8859-1") -> Response:
        if not self.check_processing_completed():
            raise LgTaskNotCompletedYetException(
                "Task is not completed yet. Please, wait a few seconds and try again."
            )

        response = get(self.Retorno["Url"], allow_redirects=True)
        response.raise_for_status()

        if encoding:
            response.encoding = encoding

        return response

    def file_as_bytes(self) -> bytes:
        return self.request_file().content

    def file_as_string_io(self, encoding: str = "ISO-8859-1") -> StringIO:
        file_text = self.request_file(encoding=encoding).text
        return StringIO(file_text)

    def file_as_bytes_io(self) -> BytesIO:
        return BytesIO().read(self.file_as_bytes())

    def download_file_locally(self, file_path: str = None) -> str:
        file_response = self.request_file()

        if not file_path:
            match = re.search(
                r"filename=(.+)$", file_response.headers.get("Content-Disposition")
            )

            if not match:
                raise ValueError("Error to define filename to this report.")

            file_path = match.group(1)

        with open(file_path, "wb") as f:
            f.write(file_response.content)

        return file_path


@dataclass
class LgApiAsyncExecutionReturn(BaseLgApiReturn):
    IdTarefa: str
    report_service_class: InitVar[BaseLgServiceClient] = None

    def __post_init__(self, report_service_class):
        self.report_service_class = report_service_class
        super().__post_init__()

    def wait_to_complete_process(
        self, time_window: int = 60, timeout: int = None
    ) -> LgApiAsyncConsultReturn:
        start_execution = time()
        async_consult = self.report_service_class.consult_task(task_id=self.IdTarefa)

        while not async_consult.check_processing_completed():
            if timeout:
                if (time() - start_execution) > timeout:
                    raise Exception(
                        f"Timeout when wait to complete process of task '{self.IdTarefa}'"
                    )

            print(
                f"Report not completed yet. Waiting {time_window} seconds to check again..."
            )
            sleep(time_window)
            async_consult = self.report_service_class.consult_task(self.IdTarefa)

        print(
            f"Report successfully generated. Blob ulr: {async_consult.Retorno['Url']}"
        )
        return async_consult



from typing import Union
from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import (
    LgApiPaginationReturn,
    LgApiReturn,
    LgApiExecReturn
)
from lg_payroll_api.helpers.base_client import (
    BaseLgServiceClient,
    LgAuthentication
)
from lg_payroll_api.utils.enums import EnumCampoContato
from lg_payroll_api.utils.enums import SITUATIONS
from lg_payroll_api.utils.lg_exceptions import LgParameterListLimitException
from datetime import date


class LgApiEmployee(BaseLgServiceClient):
    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v2/ServicoDeColaborador")

    def additional_information_list(
        self, contract_code: str, company_code: int
    ) -> LgApiReturn:
        body = {
            "Colaborador": {
                "Matricula": contract_code,
                "Empresa": {"Codigo": company_code},
            }
        }

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaValorDaInformacaoAdicionalDaPessoa,
                    body=body,
                )
            )
        )

    def consult(self, person_id: str) -> LgApiReturn:
        body = {"PessoaId": person_id}
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.Consultar, body=body
                )
            )
        )

    def documents_with_identification(
        self,
        company_code: int,
        contracts_codes: list[str] = None,
        situations: list[SITUATIONS] = None,
        page: int = None,
    ) -> LgApiPaginationReturn:
        body = {
            "CodigoDaEmpresa": company_code,
            "ListaDeMatriculas": [{"string": contract} for contract in contracts_codes]
            if contracts_codes
            else None,
            "TiposDeSituacoes": [{"int": situation} for situation in situations]
            if situations
            else None,
            "PaginaAtual": page,
        }
        return LgApiPaginationReturn(
            auth=self.lg_client,
            wsdl_service=self.wsdl_client,
            service_client=self.wsdl_client.service.ConsultarDocumentosComIdentificacao,
            body=body,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarDocumentosComIdentificacao,
                    body=body,
                )
            )
        )

    def consult_list(self, person_ids: list[str]) -> LgApiReturn:
        if len(person_ids) > 50:
            raise LgParameterListLimitException(
                "Person ids list has exceeded the limit of 50 items."
            )

        body = {"filtro": {"ListaDePessoaId": {"string": person_ids}}}

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarLista,
                    body=body,
                    parse_body_on_request=True,
                )
            )
        )

    def consult_summary_list(self, person_ids: list[str]) -> LgApiReturn:
        if len(person_ids) > 50:
            raise LgParameterListLimitException(
                "Person ids list has exceeded the limit of 50 items."
            )

        body = {"ListaDePessoaId": [{"string": person_id} for person_id in person_ids]}

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaResumida,
                    body=body,
                )
            )
        )

    def update_contact(
        self,
        contract_code: str,
        company_code: int,
        fields_of_contract: list[tuple[EnumCampoContato, str]],
        historic_date: Union[date, None] = None
    ) -> LgApiExecReturn:
        """Change contact fields in employee contract

        Args:
            contract_code (str, mandatory): Enrollment code of the contract
            company_code (int, mandatory): Company code
            fields_of_contract (list[tuple[int, str]], mandatory): List of tuples containing (EnumCampoContato, "new value")
            historic_date (date, Optional): Date of historic. If None, the current historic will be considered

        Examples:
            ```python
            # Import LG Client
            from lg_payroll_api import LgPayrollApi

            # Import the enum of contact fields
            from lg_payroll_api.utils.enums import EnumCampoContato


            lg_client = LgPayrollApi(auth)

            # Set contact fields to change
            fields_to_change = [
                (EnumCampoContato.EMAIL_CORPORATIVO, "email@domain.com"),
                (EnumCampoContato.CELULAR, "911111111")
            ]

            # Send request to LG system
            lg_client.employee_service.update_contact(
                contract_code="1",
                company_code=1,
                fields_of_contract=fields_to_change
            )
            ```
        """
        def contact_fields() -> list[dict]:
            return [
                {
                    "CampoDoContato": {
                        "CampoContato": int(field),
                        "Valor": value
                    }
                } for field, value in fields_of_contract
            ]

        body = {
            "Colaborador": {
                "Matricula": contract_code,
                "Empresa": {
                    "Codigo": company_code
                }
            },
            "DataDoHistorico": (
                historic_date.strftime("%Y-%m-%d")
                if historic_date else None
            ),
            "CamposDoContato": contact_fields()
        }

        return LgApiExecReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.AtualizarContato,
                    body=body,
                )
            )
        )



from typing import Literal

from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import (
    LgApiExecutionReturn,
    LgApiPaginationReturn,
)
from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication


class LgApiCostCenterClient(BaseLgServiceClient):
    """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

    Default class to connect with the center cost endpoints
    """

    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v2/ServicoDeCentroDeCusto")

    def list_on_demand(
        self,
        cc_code: str = None,
        permission_to_register: Literal[0, 1] = None,
        only_actives: Literal[0, 1] = None,
        company_code: int = None,
        page: int = None,
    ) -> LgApiPaginationReturn:
        """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

        Endpoint to get all center cost on LG System

        Return

        A List of OrderedDict that represents an Object(RetornoDeConsultaLista<CentroDeCusto>) API response
            [
                Tipo : int
                Mensagens : [string]
                CodigoDoErro : string
                CentroDeCusto : Object(CentroDeCusto)
            ]
        """
        params = {
            "TermoDaBusca": cc_code,
            "ComPermissaoParaCadastrarColaborador": permission_to_register,
            "SomenteAtivos": only_actives,
            "Empresa": company_code,
            "PaginaAtual": page,
        }
        return LgApiPaginationReturn(
            auth=self.lg_client,
            wsdl_service=self.wsdl_client,
            service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
            body=params,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
                    body=params,
                )
            )
        )

    def save(
        self,
        cost_center_inital_date: str,
        companies_code: list,
        cost_center_status: int,
        cost_center_description: str,
        cost_center_code: int,
        cost_center_allow_add_employee: int,
    ) -> LgApiExecutionReturn:
        """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

        Endpoint to create a center cost on LG System

        Returns:

        A OrderedDict that represents an Object(RetornoDeExecucao<CentroDeCustoV2>) API response
                {
                    OperacaoExecutada: : int
                    Codigo : string
                    CodigoDeIntegracao : string
                    Tipo : int
                    Mensagens : [string]
                    CodigoDoErro : string
                    CentroDeCustoV2 : Object(CentroDeCustoV2)
                }
        """
        return LgApiExecutionReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.Salvar,
                    body={
                        "DataInicio": cost_center_inital_date,
                        "Empresas": companies_code,
                        "Status": cost_center_status,
                        "Descricao": cost_center_description,
                        "Codigo": cost_center_code,
                        "PermiteCadastrarColaborador": cost_center_allow_add_employee,
                    },
                )
            )
        )



from typing import Literal

from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import (
    LgApiPaginationReturn
)
from lg_payroll_api.helpers.base_client import (
    BaseLgServiceClient,
    LgAuthentication
)


class LgApiRoleClient(BaseLgServiceClient):
    """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

    Default class to connect with the role endpoints, service "v2/ServicoDeCargo"
    """

    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v2/ServicoDeCargo")

    def consult_list_by_demand(
        self,
        get_aditional_information: Literal[0, 1] = None,
        get_qualifications: Literal[0, 1] = None,
        code: str = None,
        company_code: int = None,
        only_with_perm_to_reg_emp: Literal[0, 1] = None,
        only_actives: Literal[0, 1] = None,

    ) -> LgApiPaginationReturn:
        """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

        Endpoint to retrieve role on LG System

        Return

        A List of OrderedDict that represents an Object(RetornoDeConsultaListaPorDemanda<CargoCompletoV2>) API response
            [
                Tipo : int
                Mensagens : [string]
                CodigoDoErro : string
                Cargo : Object(Cargo)
            ]
        """

        params = {"filtro": {
            "FiltroPorDemanda": {
                "TermoDaBusca": code,
                "Empresa": {"Codigo": company_code},
                "SomenteComPermissaoParaCadastrarColaborador": only_with_perm_to_reg_emp,
                "SomenteAtivos": only_actives,
            },
            "EhParaConsultarInformacaoAdicional": get_aditional_information,
            "EhParaConsultarHabilitacoes": get_qualifications
        }}

        return LgApiPaginationReturn(
            auth=self.lg_client,
            wsdl_service=self.wsdl_client,
            service_client=self.wsdl_client.service.ConsultarListaCargoCompletoPorDemanda,
            body=params,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaCargoCompletoPorDemanda,
                    body=params,
                    parse_body_on_request=True
                )
            )
        )



from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import LgApiReturn, LgApiExecutionReturn
from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication
from lg_payroll_api.utils.enums import (
    EnumTipoEntidadeInformacaoAdicional, EnumIdentificadorInformacaoAdicional
)


class LgApiAdditionalInformationValueClient(BaseLgServiceClient):
    """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

    Default class to connect with the additional information value endpoints
    """
    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(
            lg_auth=lg_auth, wsdl_service="v1/ServicoDeValorDaInformacaoAdicional"
        )

    def consult_list_concept_and_info(
        self,
        concept_type: EnumTipoEntidadeInformacaoAdicional,
        concept_codes: list[str],
        additional_informations_codes: list[str] = None
    ) -> LgApiReturn:
        """Consult additional information values filtering by concept type,
        identifiers of concepts and identifiers of additional informations.

        Args:
            **concept_type _(EnumTipoEntidadeInformacaoAdicional, mandatory)_**: Additional
            information type;
            **concept_codes _(list[str], mandatory)_**: List of concept identifiers;
            **additional_informations_codes _(list[str], optional)_**: List of identifiers
            of additional informations.
        """
        if isinstance(concept_type, EnumTipoEntidadeInformacaoAdicional):
            concept_type = concept_type.value

        params = {"filtro": {
            "TipoConceito": concept_type,
            "IdentificadoresDoConceito": {"string": concept_codes},
            "IdentificadoresInformacoesAdicionais": {"string": additional_informations_codes}
        }}
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsulteListaPorConceitoEInformacao,
                    body=params,
                    parse_body_on_request=True,
                )
            )
        )

    def consult_list_by_entity(
        self,
        entity_type: EnumIdentificadorInformacaoAdicional,
        company_code: int = None,
        entity_code: int = None
    ) -> LgApiReturn:
        """**WARNING**: This method is not working yet.

        LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

        Endpoint to get list of additional informations values in LG System

        Returns:
            LgApiReturn: A List of OrderedDict that represents an Object(RetornoDeConsultaLista<ValorDaInformacaoAdicionalParcial>) API response
                [
                    Tipo : int
                    Mensagens : [string]
                    CodigoDoErro : string
                    Retorno : list[Object(ValorDaInformacaoAdicionalParcial)]
                ]
        """
        if isinstance(entity_type, EnumIdentificadorInformacaoAdicional):
            entity_type = entity_type.value

        #Get the complex type for the entity
        InfoAdic = self.wsdl_client.get_type(f"{self.lg_dto}{entity_type}")
        #Contruct the type with info
        identificador = InfoAdic(
            Codigo=entity_code,
            CodigoEmpresa=company_code
        )
        # Create the payload with the complex type
        params = {
            "filtro": {
                "Identificador": identificador
            }
        }
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaPorEntidade,
                    body=params,
                    parse_body_on_request=True,
                )
            )
        )

    # TODO fix the problem with payload sended
    def save_additional_information_value(
        self,
        additional_info_code: int,
        additional_info_value: str,
        entity_code:int,
        company_code:int,
        entity_type: EnumIdentificadorInformacaoAdicional,
    ) -> LgApiExecutionReturn:
        """**WARNING**: This method is not working yet.
        """
        if isinstance(entity_type, EnumTipoEntidadeInformacaoAdicional):
            entity_type = entity_type.value

        #Get the complex type for the entity
        InfoAdic = self.wsdl_client.get_type(f"{self.lg_dto}{entity_type}")
        #Contruct the type with info
        identificador = InfoAdic(
            Codigo=entity_code,
            CodigoEmpresa=company_code
        )
        # Create the payload with the complex type
        params = {
            "filtro": {
                "Identificador": identificador,
                "Código do conceito": additional_info_code,
                "Valor" : additional_info_value
            }
        }
        params = {
            "valores": {
                "ValorDaInformacaoAdicional": [
                    params
                ]
            }
        }
        return LgApiExecutionReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.SalvarLista,
                    body=params,
                    parse_body_on_request=True,
                )
            )
        )




from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import LgApiReturn
from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication


class LgApiCompanyClient(BaseLgServiceClient):
    """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

    Default class to connect with the companies endpoints
    """

    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v1/ServicoDeEmpresa")

    def company_list(
        self, companies_code: list[int] = None, subscriptions_code: list[str] = None
    ) -> LgApiReturn:
        """LG API INFOS https://portalgentedesucesso.lg.com.br/api.aspx

        Endpoint to get list of companies on LG System

        Returns:

        A LgApiReturn that represents an Object(RetornoDeConsultaLista<Empresa>) API response
            [
                Tipo : int
                Mensagens : [string]
                CodigoDoErro : string
                Retorno : Object(Empresa)
            ]
        """
        params = {
            "FiltroDeEmpresaPorListaDeCodigoOuInscricao": {
                "ListaDeCodigos": companies_code,
                "ListaDeInscricoes": subscriptions_code,
            }
        }

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarLista,
                    body=params,
                )
            )
        )

    def retrieve_company(
        self, company_code: int, integration_code: str = None
    ) -> LgApiReturn:
        params = {"Codigo": company_code, "CodigoDeIntegracao": integration_code}
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.Consultar,
                    body=params,
                )
            )
        )



from typing import Union

from lg_payroll_api.utils.aux_functions import read_json_file
from lg_payroll_api.utils.models import Relatorio


class ReportParameters:
    """Aux class to configure report parameters to generate reports in LG system."""

    __from_to_keys = {
        "DtoRelatorio": "Relatorio",
        "EnumTipoArquivoGerado": "TipoArquivoGerado",
        "ListaDeTiposDeArquivosDisponiveisParaGeracao": "TiposDeArquivosDisponiveisParaGeracao",
        "ListaDeGruposDeParametros": "GruposDeParametros",
        "EnumTipoGrupoDeParametrosRelatorio": "TipoGrupoDeParametrosRelatorio",
        "ListaDeParametrosDeRelatorio": "ParametrosDeRelatorio",
        "ListaDeOpcoes": "Opcoes",
        "ListaDeValorePadroes": "ValoresPadroes",
        "ListaDeValoresSelecionados": "ValoresSelecionados",
        "ListaDeGrupoDeOrdenacao": "GruposDeOrdenacao",
    }
    __rep_match_args: dict = None

    def __init__(self, parameters: Union[list[Relatorio], list[dict]]) -> None:
        self.reports: list[Relatorio] = []
        for arg in parameters:
            if isinstance(arg, dict):
                self.reports.append(Relatorio(**arg))

            elif isinstance(arg, Relatorio):
                self.reports.append(arg)

            else:
                raise ValueError(
                    "Arguments must be list of dicts or list of Relatorio."
                )

    @classmethod
    def __report_match_args(cls) -> dict:
        if not cls.__rep_match_args:
            cls.__rep_match_args = Relatorio.match_args()

        return cls.__rep_match_args

    @classmethod
    def __search_match_key(cls, search_key: str, match_args: dict = None) -> list:
        sub_match_args = (
            match_args.get(search_key, match_args)
            if match_args
            else cls.__report_match_args().get(search_key, cls.__report_match_args())
        )

        if isinstance(sub_match_args, dict):
            sub_match_args = sub_match_args[list(sub_match_args.keys())[0]]

        if search_key == "TiposDeArquivosDisponiveisParaGeracao":
            print("")
        for item in sub_match_args:
            found_key = False
            if isinstance(item, dict):
                if search_key in list(item.keys()):
                    found_key = True

                else:
                    sub_match_args = cls.__search_match_key(search_key, item)
                    if search_key in sub_match_args:
                        found_key = True

                if sub_match_args:
                    sub_match_args.append(search_key)
                break

            elif isinstance(item, str):
                found_key = item == search_key or search_key == Relatorio.__name__
                if found_key:
                    break

        if Relatorio.__name__ not in sub_match_args:
            sub_match_args.append(Relatorio.__name__)

        return sub_match_args if found_key else []

    @classmethod
    def __prepare_dict(cls, dictionary: dict, match_keys: list = None) -> dict:
        result = {}
        for key, value in dictionary.items():
            new_key = cls.__from_to_keys.get(key, key)
            new_match_keys = cls.__search_match_key(new_key)

            if new_match_keys:
                match_keys = new_match_keys
                sub_args = []

                match_keys += sub_args
                match_keys = match_keys if match_keys else []

            if new_key not in match_keys:
                continue

            if isinstance(value, dict):
                result[new_key] = cls.__prepare_dict(value, match_keys)

            elif isinstance(value, list) and value:
                if isinstance(value[0], dict):
                    result[new_key] = [
                        cls.__prepare_dict(item, match_keys) for item in value
                    ]

                else:
                    result[new_key] = value

            else:
                result[new_key] = value

        return result

    @classmethod
    def read_report_generator_json(
        cls, params: Union[list[dict], dict, str]
    ) -> "ReportParameters":
        """This method able you to use file "GereListaRelatorio.json", generated automatically into front of LG system
        when you generate a report into platform, as parameters to use in "LgReportServiceClient.generate_report"
        as the attribute report_parameters.

        Args:
            params (Union[list[dict], dict, str], mandatory): Parameters in the same format of "GereListaRelatorio.json". You
            can pass a file path if you prefer to store the json file locally to use as parameters.

        Return:
            ReportParameters
        """
        if isinstance(params, str):
            params = read_json_file(params)

        if isinstance(params, dict):
            formatted_dictionary = cls.__prepare_dict(params)

            formatted_dictionary = [formatted_dictionary]

        elif isinstance(params, list):
            formatted_dictionary = [
                cls.__prepare_dict(param)[Relatorio.__name__] for param in params
            ]

        return cls(formatted_dictionary)

    def get_formatted_parameters(self) -> list[dict]:
        """Return formatted parameters ready to be used in "LgReportServiceClient.generate_report" method."""
        return [report.as_dict() for report in self.reports]



from typing import Union

from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import (
    LgApiAsyncConsultReturn,
    LgApiAsyncExecutionReturn,
)
from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication
from lg_payroll_api.utils import ReportParameters


class LgReportServiceClient(BaseLgServiceClient):
    """Lg API report service client class to access report service endpoints.

    This class able you to use report service to generate and consult reports in Lg system.

    Reference: https://portalgentedesucesso.lg.com.br/api.aspx
    """

    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v1/ServicoDeRelatorio")

    def consult_task(self, task_id: str) -> LgApiAsyncConsultReturn:
        params = {"IdTarefa": task_id}
        return LgApiAsyncConsultReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarTarefa,
                    body=params,
                )
            )
        )

    def generate_report(
        self, company_code: int, report_parameters: Union[list[dict], ReportParameters]
    ) -> LgApiAsyncExecutionReturn:
        """Generate report with parameters configuration. You can create parameters configuration by
        following the oficial documentation, but we recommend to use the `ReportParameters` object to
        create your parameters configuration.

        Args:
            company_code (int, mandatory): Company code, this not affect filters in report.
            report_parameters (Union[list[dict], ReportParameters], mandatory): Parameters used to generate report.

        Return:
            LgApiAsyncExecutionReturn

        Usage Example:
            ```python
            # Example of parameters configuration using ReportParameters

            from lg_payroll_api import LgPayrollApi, LgAuthentication, ReportParameters

            # Set report parameters in variable
            # You can see that format in object lg_payroll_api.utils.models.Relatorio.
            # Alternatively, you can use the format automatically created when you generate a new report at front of system,
            # Getting the "GereListaRelatorio.json" file.
            parameters = [
                {
                    "Id": "123",
                    "Nome": "name.fpl",
                    "DescricaoRelatorio": "description",
                    "TipoArquivoGerado": 0,
                    "TiposDeArquivosDisponiveisParaGeracao": ["0"],
                    "GruposDeParametros": [
                        {
                            "TipoGrupoDeParametrosRelatorio": 0,
                            "Id": 0,
                            "Nome": "name",
                            "IndiceGrupo": 0,
                            "ParametrosDeRelatorio": [
                                {
                                    "IdRelatorio":"123",
                                    "Id":"id",
                                    "IndiceReal":0,
                                    "Nome":"name",
                                    "Descricao":"description",
                                    "Operacao":0,
                                    "TipoDeDado":0,
                                    "Tamanho":0,
                                    "MascaraFormatacao":"",
                                    "Opcional":True,
                                    "PodeConsultarValor":True,
                                    "PossuiDescricao":True,
                                    "CampoDefinidoPorFormula":False,
                                    "Opcoes": [],
                                    "RegrasDeAtivacao": [],
                                    "ValoresPadroes": [],
                                    "ValoresSelecionados": []
                                }
                            ]
                        }
                    ]
                }
            ]

            auth = LgAuthentication()
            lg_api = LgPayrollApi(auth)

            # Instantiate ReportParameters object passing parameters defined before
            report_params = ReportParameters(parameters)

            lg_api.report_service.generate_report(company_code=0, report_parameters=report_params)
            ```
        """
        if isinstance(report_parameters, ReportParameters):
            report_parameters = report_parameters.get_formatted_parameters()

        params = {
            "Empresa": {"Codigo": company_code},
            "Relatorios": {"Relatorio": report_parameters},
        }

        return LgApiAsyncExecutionReturn(
            report_service_class=self,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.GerarRelatorio,
                    body=params,
                    send_none_values=True,
                )
            )
        )

    def generate_report_by_name(
        self, company_code: int, report_name: str, parameters: list[str] = None
    ) -> LgApiAsyncExecutionReturn:
        """LG API INFOS

        Args:
            company_id (int, mandatory): Id of company to generate report
            report_name (str, mandatory): Name of report
            parameters (list[str], optional): List of report parameters

        Returns:

        A LgApiAsyncExecutionReturn that represents an Object(RetornoDeExecucaoAsync) API response
            [
                Tipo : int
                Mensagens : [string]
                CodigoDoErro : string
                Retorno : Object(Empresa)
            ]

        Reference: https://portalgentedesucesso.lg.com.br/api.aspx
        """
        params = {
            "Empresa": {"Codigo": company_code},
            "NomeDoRelatorio": report_name,
            "Parametros": parameters,
        }

        return LgApiAsyncExecutionReturn(
            report_service_class=self,
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.GerarRelatorioPorNome,
                    body=params,
                )
            )
        )



from enum import Enum
from typing import Literal


def get_enum_value(value: Enum) -> int:
    if isinstance(value, Enum):
        value = value.value

    return value


SITUATIONS = Literal["Afastamento", "Atividade normal", "Férias", "Recesso", "Rescisão"]
Bool = Literal[0, 1]


class EnumTipoDeDadosModificados(int, Enum):
    CONTRATUAIS = 1
    PESSOAIS = 2
    CONTRATUAIS_E_PESSOAIS = 3


class EnumTipoDeOperacao(int, Enum):
    INCLUSAO = 1
    ALTERACAO = 2
    EXCLUSAO = 3


class EnumTipoDeOperacaoContratoLog(int, Enum):
    INCLUSAO = 1
    ALTERACAO = 2
    EXCLUSAO = 3


class EnumTipoDeDadosModificadosDaUnidadeOrganizacional(int, Enum):
    DADOS_QUE_ALTERAM_HIERARQUIA = 1


class EnumTipoDeRetorno(int, Enum):
    SUCESSO = 0
    INCONSISTENCIA = 1
    ERRO = 2
    NAO_PROCESSADO = 3


class EnumOperacaoExecutada(int, Enum):
    NENHUM = 0
    OBJETO_SEM_ALTERACAO = 1
    CADASTRO = 2
    ATUALIZACAO = 3
    EXCLUSAO = 4
    CADASTRO_EM_LOTE = 5
    VALIDACAO = 6


class EnumCampoDeBuscaDoContratoDeTrabalho(int, Enum):
    MATRICULA = 0
    ID_PESSOA = 1
    CPF = 2
    IDENTIDADE = 3
    RIC = 4
    CTPS = 5
    PIS = 6
    TITULO_ELEITOR = 7
    CNH = 8


# Reports Enums


class EnumTipoArquivoRelatorio(int, Enum):
    PDF = 0
    TXT = 1
    CSV = 2


class EnumTipoGrupoDeParametrosRelatorio(int, Enum):
    PARAMETRO_DE_USUARIO = 0
    SENTENCA_SIMPLES = 1
    SENTENCA_DINAMICA = 2


class EnumOperacaoParametroRelatorio(int, Enum):
    IGUAL = 0
    DIFERENTE = 1
    MAIOR = 2
    MENOR = 3
    MAIOR_IGUAL = 4
    MENOR_IGUAL = 5
    UM_DOS_VALORES = 6
    NAO_UM_DOS_VALORES = 7


class EnumTipoDeDadoParametroRelatorio(int, Enum):
    FDT_CHAR = 0
    FDT_SHORT = 1
    FDT_INT = 2
    FDT_FLOAT = 3
    FDT_DATE_AMD = 4
    FDT_DATE_AM = 5
    FDT_BOOLEAN = 6
    FDT_DATE_SQL = 7


class EnumCampoContato(int, Enum):
    DDD_TELEFONE = 0
    TELEFONE = 1
    DDD_CELULAR = 2
    CELULAR = 3
    RAMAL = 4
    EMAIL_CORPORATIVO = 5
    EMAIL_PARTICULAR = 6
    LINKEDIN = 7
    FACEBOOK = 8
    TWITTER = 9

class EnumIdentificadorInformacaoAdicional(str ,Enum):
    CENTRO_DE_CUSTO = "InfoAdicCentroDeCusto"
    POSICAO = "InfoAdicPosicao"
    DEPARTAMENTO = "InfoAdicUnidadeOrganizacional"
    ESTABELECIMENTO = "InfoAdicEstabelecimento"
    CONTRATO_DE_TRABALHO = "InfoAdicContratoDeTrabalho"

class EnumTipoEntidadeInformacaoAdicional(int, Enum):
    NENHUM = 0
    EMPRESA = 2
    CENTRO_DE_CUSTO = 1000
    ORGAO_RESPONSAVEL = 1002
    CARGO = 1016
    DEPENDENTE = 1019
    PENSIONISTA = 1020
    SINDICATO = 1028
    PESSOA = 1029
    POSICAO = 1030
    DEPARTAMENTO = 1031
    ESTABELECIMENTO = 1032
    CONTRATO_DE_TRABALHO = 1034
    FUNCAO = 1058
    TOMADOR = 1062
    EVENTO = 1077
    FORNECEDOR = 1183
    AUTONOMO = 1194
    REQUISICAO = 3002
    CONTRATO_ATIVIDADE_APROVACAO = 3004
    VAGA = 3010
    CANDIDATO = 3014
    MOVIMENTACAO_WORKFLOW = 3020
    WKF_FERIAS = 3021
    WKF_RECESSO = 3022
    WKF_RESCISAO = 3023
    WKF_MOVIMENTACAO_CEDER = 3024
    WKF_MOVIMENTACAO_BUSCAR = 3025
    WKF_REQUISICAO = 3026
    WKF_DADOS_PESSOAIS = 3027
    WKF_DEPENDENTES = 3028
    WKF_BENEFICIOS = 3029
    ATIVIDADE = 3030
    VALE_TRANSPORTE = 3031
    WKF_AFASTAMENTO = 3032
    WKF_LANCAMENTO_DE_VALORES = 3033
    HEADCOUNT = 4011
    LINHA_DE_TRANSPORTE = 5000
    UNIDADE_DE_ENTREGA = 5002
    OKR_QUADRO = 8000
    OKR_OBJETIVO = 8001
    OKR_KEYRESULT = 8002
    OKR_INICIATIVA = 8003


class EnumTipoDeInformacaoAdicional(int, Enum):
    TEXTO = 0
    VERDADEIRO_OU_FALSO = 1
    NUMERO = 2
    MOEDA = 3
    DATA = 4
    LISTA_SELECAO_UNICA = 5
    LISTA_MULTIPLA_SELECAO = 6


class EnumTipoStatus(int, Enum):
    ATIVO = 0
    INATIVO = 1
    INATIVACAO_PROGRAMADA = 2
    RESTRITO = -1111


class EnumTipoPonto(int, Enum):
    Manual = 1
    Eletronico = 2


class EnumGrauDeInstrucao(int, Enum):
    NENHUM = 0
    ANALFABETO = 1
    ATEQUARTASERIEINCOMPLETAENSINOFUNDAMENTAL = 2
    QUARTASERIECOMPLETAENSINO = 3
    QUINTAOITAVAENSINOFUNDAMENTAL = 4
    ENSINOFUNDAMENTALCOMPLETO = 5
    ENSINOMEDIOINCOMPLETO = 6
    ENSINOMEDIOCOMPLETO = 7
    EDUCACAOSUPERIORINCOMPLETA = 8
    EDUCACAOSUPERIORCOMPLETA = 9
    POSGRADUACAOCOMPLETA = 10
    DOUTORADOCOMPLETO = 11
    SEGUNDO_GRAU_TECNICO_INCOMPLETO = 12
    SEGUNDO_GRAU_TECNICO_COMPLETO = 13
    MESTRADOCOMPLETO = 14
    POS_DOUTORADO = 15


class EnumMesesExperiencia(int, Enum):
    TRES_MESES = 3
    SEIS_MESES = 6
    NOVE_MESES = 9
    UM_ANO = 12
    DOIS_ANOS = 24
    TRES_ANOS = 36
    QUATRO_ANOS = 48
    CINCO_ANOS = 60
    SEIS_ANOS = 72
    SETE_ANOS = 84
    OITO_ANOS = 96
    NOVE_ANOS = 108
    DEZ_ANOS = 120
    QUINZE_ANOS = 180
    VINTE_ANOS = 240


class EnumModeloPosicao(int, Enum):
    POSICAO_TEMPORARIA = 0
    DUPLICAR_POSICAO = 1


class EnumTipoDeAdmissao(int, Enum):
    ADMISSAO = 1
    TRANSFERENCIA_DE_EMPRESA_DO_MESMO_GRUPO = 2
    TRANSFERENCIA_DE_EMPRESA_CONSORCIADA = 3
    TRANSFERENCIA_POR_MOTIVO_DE_SUCESSÃO = 4
    MUDANCA_DE_CPF = 6
    TRANSFERENCIA_DE_EMPRESA_SUCEDIDA_INAPTA = 7


class EnumAgenteNocivo(int, Enum):
    TRABALHADOR_NUNCA_ESTEVE_EXPOSTO = 0
    NAO_EXPOSTO_UNICO_VINCULO = 1
    EXPOSTO_APOSENTADORIA_EM_15_ANOS_DE_SERVICO = 2
    EXPOSTO_APOSENTADORIA_EM_20_ANOS_DE_SERVICO = 3
    EXPOSTO_APOSENTADORIA_EM_25_ANOS_DE_SERVICO = 4
    NAO_EXPOSTO_MULTIPLOS_VINCULOS = 5
    EXPOSTO_APOSENTADORIA_EM_15_ANOS_DE_SERVICO_MULTIPLOS_VINCULOS = 6
    EXPOSTO_APOSENTADORIA_EM_20_ANOS_DE_SERVICO_MULTIPLOS_VINCULOS = 7
    EXPOSTO_APOSENTADORIA_EM_25_ANOS_DE_SERVICO_MULTIPLOS_VINCULOS = 8


class EnumFiliacaoSindical(int, Enum):
    NAO_FILIADO = 0
    FILIADO_AO_SINDICATO = 1
    FILIADO_AO_SINDICATO_PROFICIONAL_LIBERAL = 2
    FILIADO_EM_AMBOS_OS_SINDICATOS = 3


class EnumFormaDePagamento(int, Enum):
    NENHUM = 0
    CREDITO_EM_CONTA_CORRENTE = 1
    CARTAO_SALARIO = 2
    CARTAO_SAQUE = 3
    CHEQUE_ADMINISTRATIVO = 4
    CONTA_EXPRESSA = 5
    CREDITO_EM_CONTA_POUPANCA = 6
    CREDITO_EM_CONTA_REAL_TIME = 7
    PAGAMENTO_DOC = 8
    FICHA_DE_COMPENSACAO = 9
    ORDEM_DE_PAGAMENTO = 10
    RECIBO = 11
    PAGAMENTO_TED = 12
    PAGAMENTO_DINHEIRO = 13



from typing import Optional
from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import (
    LgApiReturn,
)
from lg_payroll_api.helpers.base_client import BaseLgServiceClient, LgAuthentication


class LgApiWorkScaleClient(BaseLgServiceClient):
    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v2/ServicoDeEscala")

    def consult_list_on_demand(
        self,
        permission_filter: Optional[bool] = None,
        search_term: Optional[str] = None,
        only_actives: Optional[bool] = None,
        scale_code: Optional[str] = None,
        current_page: Optional[int] = 1,
    ) -> LgApiReturn:
        body = {
            "FiltroDeEscala": {
                "SomenteComPermissaoParaCadColaborador": permission_filter,
                "TermoDaBusca": search_term,
                "SomenteAtivos": only_actives,
                "CodigoDaEscala": scale_code,
                "PaginaAtual": current_page,
            }
        }

        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaPorDemanda,
                    body=body,
                )
            )
        )



from typing import Union
from zeep.helpers import serialize_object

from lg_payroll_api.helpers.api_results import (
    LgApiReturn,
    LgApiSaveListReturn
)
from lg_payroll_api.helpers.base_client import (
    BaseLgServiceClient,
    LgAuthentication
)
from lg_payroll_api.utils.enums import EnumTipoDeAdmissao, EnumAgenteNocivo, EnumFiliacaoSindical, EnumFormaDePagamento, EnumTipoPonto
from lg_payroll_api.utils.lg_exceptions import LgParameterListLimitException
from datetime import date
from lg_payroll_api.utils.aux_functions import bool_to_int


class LgApiMovementClient(BaseLgServiceClient):
    """
    Client for interacting with the LG Payroll API's movement (employee transfer and update) services.
    This class provides methods to:
    - Consult the historic list of movements for a single contract, with various optional filters.
    - Consult the historic list of movements for multiple contracts within a company and date range.
    - Register (save) a movement for a contract, supporting a wide range of movement types and related data.
    - Delete a movement for a contract by contract code, company code, and occurrence date.

    Inheritance:
        BaseLgServiceClient: Provides base functionality for LG API service clients.
        lg_auth (LgAuthentication): Authentication object for accessing the LG API.

    Methods:
        consult_historic_list(...): Consults the historic list of a contract with optional search filters.
        consult_list_for_multiple_contracts(...): Consults the historical list for multiple contracts.
        save_movement(...): Registers a movement (transfer, update, or change) for a contract.
        delete_movement(...): Deletes a movement for a contract.

    Returns:
        LgApiReturn, LgApiSaveListReturn, or LgApiExecReturn: Depending on the operation performed.

    Raises:
        LgParameterListLimitException: If the number of contract codes exceeds the allowed limit in consult_list_for_multiple_contracts.
    """

    def __init__(self, lg_auth: LgAuthentication):
        super().__init__(lg_auth=lg_auth, wsdl_service="v1/ServicoDeMovimentacao")
    
    def consult_historic_list(
        self,
        contract_code: str,
        company_code: int,
        start_date: Union[date, None] = None,
        end_date: Union[date, None] = None,
        search_company: bool = None,
        search_position: bool = None,
        search_office: bool = None,
        search_cost_center: bool = None,
        search_organizational_unit: bool = None,
        search_permanent_role: bool = None,
        search_commissioned_role: bool = None,
        search_tax_jurisdiction: bool = None,
        search_responsible_authority: bool = None,
        search_professional_nature: bool = None,
        search_union: bool = None,
        search_professional_liberal_union: bool = None,
        search_harmful_agent: bool = None,
        search_effective_function: bool = None,
        search_commissioned_function: bool = None,
        search_salarial_table: bool = None,
        search_work_shift: bool = None,
        search_banking_data: bool = None,
        search_salary_adjustment: bool = None,
        search_contract_type: bool = None,
    ) -> LgApiReturn:
        """
        Consults the historic list of a contract with various optional search filters.

        Parameters:
            contract_code (str): The contract's unique identifier (matricula).
            company_code (int): The code of the company associated with the contract.
            start_date (Union[date, None], optional): The start date for the search period. Defaults to None.
            end_date (Union[date, None], optional): The end date for the search period. Defaults to None.
            search_company (bool, optional): Whether to include company information in the search. Defaults to False.
            search_position (bool, optional): Whether to include position information in the search. Defaults to False.
            search_office (bool, optional): Whether to include office/establishment information in the search. Defaults to False.
            search_cost_center (bool, optional): Whether to include cost center information in the search. Defaults to False.
            search_organizational_unit (bool, optional): Whether to include organizational unit information in the search. Defaults to False.
            search_permanent_role (bool, optional): Whether to include permanent role information in the search. Defaults to False.
            search_commissioned_role (bool, optional): Whether to include commissioned role information in the search. Defaults to False.
            search_tax_jurisdiction (bool, optional): Whether to include tax jurisdiction information in the search. Defaults to False.
            search_responsible_authority (bool, optional): Whether to include responsible authority information in the search. Defaults to False.
            search_professional_nature (bool, optional): Whether to include professional nature information in the search. Defaults to False.
            search_union (bool, optional): Whether to include union information in the search. Defaults to False.
            search_professional_liberal_union (bool, optional): Whether to include professional liberal union information in the search. Defaults to False.
            search_harmful_agent (bool, optional): Whether to include harmful agent information in the search. Defaults to False.
            search_effective_function (bool, optional): Whether to include effective function information in the search. Defaults to False.
            search_commissioned_function (bool, optional): Whether to include commissioned function information in the search. Defaults to False.
            search_salarial_table (bool, optional): Whether to include salary table information in the search. Defaults to False.
            search_work_shift (bool, optional): Whether to include work shift information in the search. Defaults to False.
            search_banking_data (bool, optional): Whether to include banking data in the search. Defaults to False.
            search_salary_adjustment (bool, optional): Whether to include salary adjustment information in the search. Defaults to False.
            search_contract_type (bool, optional): Whether to include contract type information in the search. Defaults to False.

        Returns:
            LgApiReturn: The result of the API call, containing the historic list data according to the specified filters.
        """
        body = {
            "IdentificacaoDeContrato": {
                "Matricula": contract_code,
                "Empresa": {"Codigo": company_code},
            },
            "Periodo": {
                "DataInicio": (
                    start_date.strftime("%Y-%m-%d") if start_date else None
                ),
                "DataFim": (
                    end_date.strftime("%Y-%m-%d") if end_date else None
                ),
            },
            "EmpresaBuscar": bool_to_int(search_company),
            "PosicaoBuscar": bool_to_int(search_position),
            "EstabelecimentoBuscar": bool_to_int(search_office),
            "CentroDeCustoBuscar": bool_to_int(search_cost_center),
            "UnidadeOrganizacionalBuscar": bool_to_int(search_organizational_unit),
            "CargoEfetivoBuscar": bool_to_int(search_permanent_role),
            "CargoComissionadoBuscar": bool_to_int(search_commissioned_role),
            "LotacaoTributariaBuscar": bool_to_int(search_tax_jurisdiction),
            "OrgaoResponsavelBuscar": bool_to_int(search_responsible_authority),
            "NaturezaProfissionalBuscar": bool_to_int(search_professional_nature),
            "SindicatoBuscar": bool_to_int(search_union),
            "SindicatoProfissionalLiberalBuscar": bool_to_int(search_professional_liberal_union),
            "AgenteNocivoBuscar": bool_to_int(search_harmful_agent),
            "FuncaoEfetivaBuscar": bool_to_int(search_effective_function),
            "FuncaoComissionadaBuscar": bool_to_int(search_commissioned_function),
            "TabelaSalarialBuscar": bool_to_int(search_salarial_table),
            "EscalaBuscar": bool_to_int(search_work_shift),
            "DadosBancariosBuscar": bool_to_int(search_banking_data),
            "ReajusteSalarialBuscar": bool_to_int(search_salary_adjustment),
            "ModalidadeContratualBuscar": bool_to_int(search_contract_type),
        }
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaHistorico,
                    body=body,
                    parse_body_on_request=False,
                )
            )
        )
    
    def consult_list_for_multiple_contracts(
        self,
        company_code: int,
        contracts_codes: list[str],
        start_date: Union[date, None] = None,
        end_date: Union[date, None] = None,
    ) -> LgApiReturn:
        """
        Consults the historical list for multiple contracts within a specified company and date range.
        Args:
            company_code (int): The code of the company to query.
            contracts_codes (list[str], mandatory): A list of contract codes (matriculas) to consult. 
                The maximum allowed is 50.
            start_date (Union[date, None], optional): The start date for the period to consult. Defaults to None.
            end_date (Union[date, None], optional): The end date for the period to consult. Defaults to None.
        Returns:
            LgApiReturn: The result of the API call, containing the historical data for the specified contracts.
        Raises:
            LgParameterListLimitException: If the number of contract codes exceeds the allowed limit.
        """
        limit_contracts: int = 50
        if contracts_codes and len(contracts_codes) > limit_contracts:
            raise LgParameterListLimitException(
                f"The maximum number of contracts is {limit_contracts}."
            )
        
        body = {
            "CodigoDaEmpresa": company_code,
            "ListaDeMatriculas": [{"string": contract} for contract in contracts_codes],
            "Periodo": {
                "DataInicio": (
                    start_date.strftime("%Y-%m-%d") if start_date else None
                ),
                "DataFim": (
                    end_date.strftime("%Y-%m-%d") if end_date else None
                ),
            },
        }
        return LgApiReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ConsultarListaParaVariosContratos,
                    body=body,
                )
            )
        )

    def save_movement(
        self,
        contract_code: str,
        company_code: int,
        occurrence_date: date,
        description: str,
        reason_code: int,
        new_company_code: int = None,
        use_current_contract_code: bool = None,
        contract_code_in_new_company: str = None,
        contract_code_esocial_new_company: str = None,
        transfer_employee_historic: bool = None,
        admission_type_esocial: EnumTipoDeAdmissao = None,
        new_position_code: str = None,
        new_office_code: str = None,
        new_tax_jurisdiction_code: str = None,
        new_responsible_authority_code: str = None,
        new_organizational_unit_code: str = None,
        movement_even_if_the_colaborator_is_manager: bool = None,
        new_cost_center_code: str = None,
        new_effective_role_code: str = None,
        frame_new_effective_role: bool = None,
        new_salarial_table_effective_role_code: str = None,
        new_salarial_range_effective_role_code: str = None,
        new_salarial_level_effective_role_code: str = None,
        new_commissioned_role_code: str = None,
        new_salarial_table_commissioned_role_code: str = None,
        new_salarial_range_commissioned_role_code: str = None,
        new_salarial_level_commissioned_role_code: str = None,
        new_effective_function_code: str = None,
        new_salarial_table_effective_function_code: str = None,
        new_salarial_range_effective_function_code: str = None,
        new_salarial_level_effective_function_code: str = None,
        new_commissioned_function_code: str = None,
        new_salarial_table_commissioned_function_code: str = None,
        new_salarial_range_commissioned_function_code: str = None,
        new_salarial_level_commissioned_function_code: str = None,
        unlink_variable_salary: bool = None,
        variable_salary_description: str = None,
        new_harmful_agent_type: EnumAgenteNocivo = None,
        new_professional_nature_code: str = None,
        unlink_professional_nature: bool = None,
        new_union_code: str = None,
        unlink_union: bool = None,
        compare_salary_with_base_of_new_union: bool = None,
        new_professional_liberal_union_code: str = None,
        unlink_professional_liberal_union: bool = None,
        new_union_filiation_type: EnumFiliacaoSindical = None,
        unlink_union_filiation_type: bool = None,
        new_bank_agency_code: str = None,
        new_bank_code: str = None,
        new_bank_account_number: str = None,
        new_bank_account_digit: str = None,
        new_payment_method: EnumFormaDePagamento = None,
        new_account_type: str = None,
        unlink_banking_data: bool = None,
        new_work_shift_code: str = None,
        new_timekeeping_type: EnumTipoPonto = None,
        timekeeping_required: bool = None,
        new_contract_type_code: str = None,
        observations: str = None,
    ) -> LgApiSaveListReturn:
        """
        Registers a movement (transfer, update, or change) for a contract in the payroll system.

        Args:
            contract_code (str): The contract code (employee registration number).
            company_code (int): The code of the company where the contract currently exists.
            occurrence_date (date): The date when the movement occurs.
            description (str): Description of the movement.
            reason_code (int): Code representing the reason for the movement.
            new_company_code (int, optional): Code of the destination company for transfer movements.
            use_current_contract_code (bool, optional): Whether to use the same contract code in the new company.
            contract_code_in_new_company (str, optional): Contract code to be used in the new company.
            contract_code_esocial_new_company (str, optional): eSocial contract code for the new company.
            transfer_employee_historic (bool, optional): Whether to transfer the employee's history.
            admission_type_esocial (EnumTipoDeAdmissao, optional): eSocial admission type for the new company.
            new_position_code (str, optional): Code of the new position.
            new_office_code (str, optional): Code of the new office/establishment.
            new_tax_jurisdiction_code (str, optional): Code of the new tax jurisdiction.
            new_responsible_authority_code (str, optional): Code of the new responsible authority.
            new_organizational_unit_code (str, optional): Code of the new organizational unit.
            movement_even_if_the_colaborator_is_manager (bool, optional): Move even if the employee is the unit manager.
            new_cost_center_code (str, optional): Code of the new cost center.
            new_effective_role_code (str, optional): Code of the new effective role.
            frame_new_effective_role (bool, optional): Whether to frame the new effective role.
            new_salarial_table_effective_role_code (str, optional): Code of the new salary table for the effective role.
            new_salarial_range_effective_role_code (str, optional): Code of the new salary range for the effective role.
            new_salarial_level_effective_role_code (str, optional): Code of the new salary level for the effective role.
            new_commissioned_role_code (str, optional): Code of the new commissioned role.
            new_salarial_table_commissioned_role_code (str, optional): Code of the new salary table for the commissioned role.
            new_salarial_range_commissioned_role_code (str, optional): Code of the new salary range for the commissioned role.
            new_salarial_level_commissioned_role_code (str, optional): Code of the new salary level for the commissioned role.
            new_effective_function_code (str, optional): Code of the new effective function.
            new_salarial_table_effective_function_code (str, optional): Code of the new salary table for the effective function.
            new_salarial_range_effective_function_code (str, optional): Code of the new salary range for the effective function.
            new_salarial_level_effective_function_code (str, optional): Code of the new salary level for the effective function.
            new_commissioned_function_code (str, optional): Code of the new commissioned function.
            new_salarial_table_commissioned_function_code (str, optional): Code of the new salary table for the commissioned function.
            new_salarial_range_commissioned_function_code (str, optional): Code of the new salary range for the commissioned function.
            new_salarial_level_commissioned_function_code (str, optional): Code of the new salary level for the commissioned function.
            unlink_variable_salary (bool, optional): Whether to unlink the variable salary.
            variable_salary_description (str, optional): Description for the variable salary.
            new_harmful_agent_type (EnumAgenteNocivo, optional): Type of harmful agent for the new position.
            new_professional_nature_code (str, optional): Code of the new professional nature.
            unlink_professional_nature (bool, optional): Whether to unlink the professional nature.
            new_union_code (str, optional): Code of the new union.
            unlink_union (bool, optional): Whether to unlink the union.
            compare_salary_with_base_of_new_union (bool, optional): Compare salary with the base of the new union.
            new_professional_liberal_union_code (str, optional): Code of the new professional liberal union.
            unlink_professional_liberal_union (bool, optional): Whether to unlink the professional liberal union.
            new_union_filiation_type (EnumFiliacaoSindical, optional): Type of new union filiation.
            unlink_union_filiation_type (bool, optional): Whether to unlink the union filiation type.
            new_bank_agency_code (str, optional): Code of the new bank agency.
            new_bank_code (str, optional): Code of the new bank.
            new_bank_account_number (str, optional): New bank account number.
            new_bank_account_digit (str, optional): New bank account digit.
            new_payment_method (EnumFormaDePagamento, optional): New payment method.
            new_account_type (str, optional): New account type.
            unlink_banking_data (bool, optional): Whether to unlink banking data.
            new_work_shift_code (str, optional): Code of the new work shift.
            new_timekeeping_type (EnumTipoPonto, optional): New timekeeping type.
            timekeeping_required (bool, optional): Whether timekeeping is required.
            new_contract_type_code (str, optional): Code of the new contract type.
            observations (str, optional): Additional observations.

        Returns:
            LgApiExecReturn: The result of the movement registration operation.
        """
        factory_moved_items = self.wsdl_client.type_factory("ns1")
        moved_items = []
        if new_company_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoEmpresa(
                CodigoDaEmpresaDestino=new_company_code,
                UtilizarAMesmaMatriculaDaEmpresaDeOrigemNaEmpresaDestino=bool_to_int(use_current_contract_code),
                MatriculaNaEmpresaDestino=contract_code_in_new_company,
                MatriculaEsocialNaEmpresaDestino=contract_code_esocial_new_company,
                TransferirHistoricoDoColaborador=bool_to_int(transfer_employee_historic),
                TipoDeAdmissaoEsocial=(
                    admission_type_esocial.value
                    if admission_type_esocial else None
                ))
            )
        
        if new_position_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoPosicao(
                    Codigo=new_position_code,
                )
            )
        if new_office_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoEstabelecimento(
                Codigo=new_office_code,
            ))
        if new_tax_jurisdiction_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoLotacaoTributaria(
                Codigo=new_tax_jurisdiction_code,
            ))
        if new_responsible_authority_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoOrgaoResponsavel(
                Codigo=new_responsible_authority_code,
            ))
        if new_organizational_unit_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoUnidadeOrganizacional(
                Codigo=new_organizational_unit_code,
                MovimentarMesmoQueColaboradorSejaGestorDaUnidade=bool_to_int(movement_even_if_the_colaborator_is_manager),
            ))
        if new_cost_center_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoCentroDeCusto(
                Codigo=new_cost_center_code,
            ))
        if new_effective_role_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoCargoEfetivo(
                Codigo=new_effective_role_code,
                EnquadrarCargoEfetivo=bool_to_int(frame_new_effective_role),
            ))
        if new_salarial_table_effective_role_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoTabelaSalarialCargoEfetivo(
                Codigo=new_salarial_table_effective_role_code,
            ))
        if new_salarial_range_effective_role_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoFaixaSalarialCargoEfetivo(
                Codigo=new_salarial_range_effective_role_code,
            ))
        if new_salarial_level_effective_role_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoNivelSalarialCargoEfetivo(
                Codigo=new_salarial_level_effective_role_code,
            ))
        if new_commissioned_role_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoCargoComissionado(
                Codigo=new_commissioned_role_code,
            ))
        if new_salarial_table_commissioned_role_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoTabelaSalarialCargoComissionado(
                Codigo=new_salarial_table_commissioned_role_code,
            ))
        if new_salarial_range_commissioned_role_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoFaixaSalarialCargoComissionado(
                Codigo=new_salarial_range_commissioned_role_code,
            ))
        if new_salarial_level_commissioned_role_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoNivelSalarialCargoComissionado(
                Codigo=new_salarial_level_commissioned_role_code,
            ))
        if new_effective_function_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoFuncaoEfetiva(
                Codigo=new_effective_function_code,
            ))
        if new_salarial_table_effective_function_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoTabelaSalarialFuncaoEfetiva(
                Codigo=new_salarial_table_effective_function_code,
            ))
        if new_salarial_range_effective_function_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoFaixaSalarialFuncaoEfetiva(
                Codigo=new_salarial_range_effective_function_code,
            ))
        if new_salarial_level_effective_function_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoNivelSalarialFuncaoEfetiva(
                Codigo=new_salarial_level_effective_function_code,
            ))
        if new_commissioned_function_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoFuncaoComissionada(
                Codigo=new_commissioned_function_code,
            ))
        if new_salarial_table_commissioned_function_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoTabelaSalarialFuncaoComissionada(
                Codigo=new_salarial_table_commissioned_function_code,
            ))
        if new_salarial_range_commissioned_function_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoFaixaSalarialFuncaoComissionada(
                Codigo=new_salarial_range_commissioned_function_code,
            ))
        if new_salarial_level_commissioned_function_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoNivelSalarialFuncaoComissionada(
                Codigo=new_salarial_level_commissioned_function_code,
            ))
        if unlink_variable_salary is not None or variable_salary_description is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoDescricaoDoSalarioVariavel(
                Desvincular=bool_to_int(unlink_variable_salary),
                Descricao=variable_salary_description,
            ))
        if new_harmful_agent_type is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoAgenteNocivo(
                AgenteNocivo=new_harmful_agent_type.value,
            ))
        if new_professional_nature_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoNaturezaProfissional(
                Codigo=new_professional_nature_code,
                Desvincular=bool_to_int(unlink_professional_nature),
            ))
        if new_union_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoSindicato(
                Codigo=new_union_code,
                Desvincular=bool_to_int(unlink_union),
                CompararSalarioComOPisoDoNovoSindicato=compare_salary_with_base_of_new_union,
            ))
        if new_professional_liberal_union_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoSindicatoProfissionalLiberal(
                Codigo=new_professional_liberal_union_code,
                Desvincular=bool_to_int(unlink_professional_liberal_union),
            ))
        if new_union_filiation_type is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoFiliacaoSindical(
                FiliacaoSindical=new_union_filiation_type.value,
                Desvincular=bool_to_int(unlink_union_filiation_type),
            ))
        if new_bank_agency_code is not None or new_bank_code is not None or \
           new_bank_account_number is not None or new_bank_account_digit is not None or \
           new_payment_method is not None or new_account_type is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoDadosBancariosV3(
                CodigoDaAgencia=new_bank_agency_code,
                CodigoDoBanco=new_bank_code,
                Conta=new_bank_account_number,
                DigitoDaConta=new_bank_account_digit,
                FormaDePagamento=(
                    new_payment_method.value if new_payment_method else None
                ),
                TipoDaConta=new_account_type,
                Desvincular=bool_to_int(unlink_banking_data),
            ))
        
        if new_work_shift_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoEscala(
                Codigo=new_work_shift_code,
            ))
        if new_timekeeping_type is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoTipoDePonto(
                TipoDePonto=new_timekeeping_type.value,
            ))
        if timekeeping_required is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoMarcarPonto(
                Valor=bool_to_int(timekeeping_required),
            ))

        if new_contract_type_code is not None:
            moved_items.append(factory_moved_items.ItemMovimentadoModalidadeContratual(
                Codigo=new_contract_type_code,
            ))

        body = {
            "MovimentacaoV2":{
                "IdentificacaoDoContrato": {
                    "Matricula": contract_code,
                    "Empresa": {"Codigo": company_code},
                },
                "DataDaOcorrencia": occurrence_date.strftime("%Y-%m-%d"),
                "ParametrosDeMovimentacao": {
                    "Descricao": description,
                    "Motivo": {"Codigo": reason_code},
                    "Observacoes": observations,
                },
                "ItensMovimentados": {
                    "ItemMovimentadoAbstratoV2": moved_items
                },
            }
        }
        return LgApiSaveListReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.CadastrarLista,
                    body=body,
                )
            )
        )

    def delete_movement(
        self,
        contract_code: str,
        company_code: int,
        occurrence_date: date,
    ) -> LgApiSaveListReturn:
        """
        Deletes a movement for a contract in the payroll system.

        Args:
            contract_code (str): The contract code (employee registration number).
            company_code (int): The code of the company where the contract currently exists.
            occurrence_date (date): The date when the movement occurs.

        Returns:
            LgApiExecReturn: The result of the movement deletion operation.
        """
        body = {
            "IdentificacaoDoContrato": {
                "IdentificacaoDeContrato": {
                    "Matricula": contract_code,
                    "Empresa": {"Codigo": company_code},
                },
                "DataOcorrencia": occurrence_date.strftime("%Y-%m-%d"),
            }
        }
        return LgApiSaveListReturn(
            **serialize_object(
                self.send_request(
                    service_client=self.wsdl_client.service.ExcluirLista,
                    body=body,
                )
            )
        )



from dataclasses import dataclass, field, fields, is_dataclass
from datetime import date, datetime

from zeep.client import Factory

from lg_payroll_api.utils.enums import (
    Bool,
    EnumOperacaoParametroRelatorio,
    EnumTipoArquivoRelatorio,
    EnumTipoDeDadoParametroRelatorio,
    EnumTipoGrupoDeParametrosRelatorio,
)


class ServiceParametersAdapter:
    """This class contains methods that help to transform dataclasses that represent
    API objects, in dictionaries, respecting the object nomenclature.
    """

    @classmethod
    def match_args(cls) -> dict:
        match_arguments: list = [_field.name for _field in fields(cls)]
        parameters = cls.__annotations__
        result: dict = {cls.__name__: match_arguments}

        for key, value in parameters.items():
            args = getattr(value, "__args__", None)
            args = args[0] if args else None

            if is_dataclass(value):
                if hasattr(value, "match_args"):
                    match_arguments.append(value.match_args())

            elif is_dataclass(args):
                if hasattr(args, "match_args"):
                    match_arguments.append(args.match_args())

        return result

    def as_dict(self, obj_factory: Factory = None) -> dict:
        """Generate dictionary of objects configured to use as report parameters"""
        parameters = self.__class__.__annotations__
        record = {}
        for key, value in parameters.items():
            param_value = self.__dict__[key]
            args = getattr(value, "__args__", None)
            args = args[0] if args else None

            if is_dataclass(value) and param_value:
                if hasattr(param_value, "as_dict"):
                    record[key] = param_value.as_dict()

                else:
                    record[key] = param_value

            elif isinstance(param_value, list) and param_value:
                record_list = []

                for val in param_value:
                    if hasattr(val, "as_dict"):
                        record_list.append(val.as_dict())
                        list_key = val.__class__.__name__

                    else:
                        record_list.append(val)
                        if isinstance(val, str):
                            list_key = "string"

                        else:
                            list_key = val.__class__.__name__

                record[key] = {list_key: record_list}

            else:
                record[key] = param_value

        if obj_factory:
            complexType = getattr(obj_factory, self.__class__.__name__)
            result = complexType(**record)

        else:
            result = record

        return result


@dataclass
class NestedDataClass:
    """Aux dataclass to work with dataclasses attributes into anothers dataclasses"""

    def __post_init__(self):
        self.__nested_objs()

    def __nested_objs(self):
        parameters = self.__class__.__annotations__
        for key, value in parameters.items():
            args = getattr(value, "__args__", None)
            args = args[0] if args else None
            param_value = self.__dict__[key]

            if is_dataclass(value) or is_dataclass(args):
                if isinstance(param_value, dict):
                    self.__dict__[key] = value(**param_value)

                elif isinstance(param_value, list):
                    self.__dict__[key] = [args(**item) for item in param_value]

            if value == date and isinstance(param_value, datetime):
                self.__dict__[key] = param_value.date()


@dataclass
class ValorParametro(ServiceParametersAdapter):
    IdentificacaoValorDoParametro: int = None
    Valor: int = None
    Descricao: int = None


@dataclass
class RegraDeAtivacaoDoParametro(ServiceParametersAdapter):
    IdentificadorDoParametro: str = None
    Operador: EnumOperacaoParametroRelatorio = None
    ListaDeValores: list[str] = field(default_factory=list)


@dataclass
class ParametroDeRelatorio(NestedDataClass, ServiceParametersAdapter):
    IdRelatorio: str
    Id: str
    IndiceReal: int
    Nome: str
    Descricao: str
    Operacao: EnumOperacaoParametroRelatorio
    TipoDeDado: EnumTipoDeDadoParametroRelatorio
    Tamanho: int
    MascaraFormatacao: str
    Opcional: Bool
    PodeConsultarValor: Bool
    PossuiDescricao: Bool
    CampoDefinidoPorFormula: Bool
    Opcoes: list[ValorParametro] = field(default_factory=list)
    RegrasDeAtivacao: list[RegraDeAtivacaoDoParametro] = field(default_factory=list)
    ValoresPadroes: list[str] = field(default_factory=list)
    ValoresSelecionados: list[ValorParametro] = field(default_factory=list)


@dataclass
class GrupoDeParametros(NestedDataClass, ServiceParametersAdapter):
    TipoGrupoDeParametrosRelatorio: EnumTipoGrupoDeParametrosRelatorio
    Id: int
    Nome: str
    IndiceGrupo: int
    ParametrosDeRelatorio: list[ParametroDeRelatorio]


@dataclass
class Relatorio(NestedDataClass, ServiceParametersAdapter):
    """Report data. Used to define report parameters."""

    Id: str
    Nome: str
    DescricaoRelatorio: str
    TipoArquivoGerado: EnumTipoArquivoRelatorio
    TiposDeArquivosDisponiveisParaGeracao: list[EnumTipoArquivoRelatorio] = field(
        default_factory=list
    )
    GruposDeParametros: list[GrupoDeParametros] = field(default_factory=list)
    GruposDeOrdenacao: list[str] = field(default_factory=list)
    GrupoDeOrdenacaoPadrao: str = None
    GrupoDeOrdenacaoSelecionado: str = None
    ExtensaoRelatorioGerado: str = None

    def __post_init__(self):
        self.TiposDeArquivosDisponiveisParaGeracao = [
            str(item) for item in self.TiposDeArquivosDisponiveisParaGeracao
        ]
        return super().__post_init__()
